#include "test.hpp"

int main(int argc, char** argv) {
  using namespace SH;

  char* last_test = "none";

  Test test(argc, argv);

  int errors = 0, total_tests = 0;
  ShProgram add_a_f_3f_3f_infix_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = a + b;
  } SH_END;

  add_a_f_3f_3f_infix_test0.name("add_a_f_3f_3f_infix_test0");
  last_test = "add_a_f_3f_3f_infix_test0";
  {  // f_3f_3f_infix_test0
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> b(float(3.0), float(4.0), float(5.0));
    ShAttrib<3, SH_CONST, float> expected(float(3.0), float(5.0), float(7.0));

    try {
      if (test.run(add_a_f_3f_3f_infix_test0, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_f_3f_1f_infix_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<1, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = a + b;
  } SH_END;

  add_a_f_3f_1f_infix_test1.name("add_a_f_3f_1f_infix_test1");
  last_test = "add_a_f_3f_1f_infix_test1";
  {  // f_3f_1f_infix_test1
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(2.0), float(3.0));
    ShAttrib<1, SH_CONST, float> b(float(7.0));
    ShAttrib<3, SH_CONST, float> expected(float(8.0), float(9.0), float(10.0));

    try {
      if (test.run(add_a_f_3f_1f_infix_test1, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_f_1f_1f_infix_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_INPUT, float> b;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = a + b;
  } SH_END;

  add_a_f_1f_1f_infix_test2.name("add_a_f_1f_1f_infix_test2");
  last_test = "add_a_f_1f_1f_infix_test2";
  {  // f_1f_1f_infix_test2
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> b(float(7.0));
    ShAttrib<1, SH_CONST, float> expected(float(8.0));

    try {
      if (test.run(add_a_f_1f_1f_infix_test2, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_f_3f_3f_infix_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = a + b;
  } SH_END;

  add_a_f_3f_3f_infix_test3.name("add_a_f_3f_3f_infix_test3");
  last_test = "add_a_f_3f_3f_infix_test3";
  {  // f_3f_3f_infix_test3
    ShAttrib<3, SH_CONST, float> a(float(4.0), float(5.0), float(6.0));
    ShAttrib<3, SH_CONST, float> b(float(-4.0), float(-5.0), float(-6.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.0), float(0.0), float(0.0));

    try {
      if (test.run(add_a_f_3f_3f_infix_test3, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_i_3i_3i_infix_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, int> a;
    ShAttrib<3, SH_INPUT, int> b;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = a + b;
  } SH_END;

  add_a_i_3i_3i_infix_test4.name("add_a_i_3i_3i_infix_test4");
  last_test = "add_a_i_3i_3i_infix_test4";
  {  // i_3i_3i_infix_test4
    ShAttrib<3, SH_CONST, int> a(int(0), int(1), int(2));
    ShAttrib<3, SH_CONST, int> b(int(3), int(4), int(5));
    ShAttrib<3, SH_CONST, int> expected(int(3), int(5), int(7));

    try {
      if (test.run(add_a_i_3i_3i_infix_test4, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_i_3s_1b_infix_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, short> a;
    ShAttrib<1, SH_INPUT, char> b;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = a + b;
  } SH_END;

  add_a_i_3s_1b_infix_test5.name("add_a_i_3s_1b_infix_test5");
  last_test = "add_a_i_3s_1b_infix_test5";
  {  // i_3s_1b_infix_test5
    ShAttrib<3, SH_CONST, short> a(short(1), short(2), short(3));
    ShAttrib<1, SH_CONST, char> b(char(7));
    ShAttrib<3, SH_CONST, int> expected(int(8), int(9), int(10));

    try {
      if (test.run(add_a_i_3s_1b_infix_test5, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_i_1i_1i_infix_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, int> a;
    ShAttrib<1, SH_INPUT, int> b;
    ShAttrib<1, SH_OUTPUT, int> out;
    out = a + b;
  } SH_END;

  add_a_i_1i_1i_infix_test6.name("add_a_i_1i_1i_infix_test6");
  last_test = "add_a_i_1i_1i_infix_test6";
  {  // i_1i_1i_infix_test6
    ShAttrib<1, SH_CONST, int> a(int(1));
    ShAttrib<1, SH_CONST, int> b(int(7));
    ShAttrib<1, SH_CONST, int> expected(int(8));

    try {
      if (test.run(add_a_i_1i_1i_infix_test6, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_i_3i_3i_infix_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, int> a;
    ShAttrib<3, SH_INPUT, int> b;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = a + b;
  } SH_END;

  add_a_i_3i_3i_infix_test7.name("add_a_i_3i_3i_infix_test7");
  last_test = "add_a_i_3i_3i_infix_test7";
  {  // i_3i_3i_infix_test7
    ShAttrib<3, SH_CONST, int> a(int(4), int(5), int(6));
    ShAttrib<3, SH_CONST, int> b(int(-4), int(-5), int(-6));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(0), int(0));

    try {
      if (test.run(add_a_i_3i_3i_infix_test7, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_fui_4f_4f_infix_test8 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, ShFracUInt> out;
    out = a + b;
  } SH_END;

  add_a_fui_4f_4f_infix_test8.name("add_a_fui_4f_4f_infix_test8");
  last_test = "add_a_fui_4f_4f_infix_test8";
  {  // fui_4f_4f_infix_test8
    ShAttrib<4, SH_CONST, float> a(float(-4), float(0.3), float(1.0), float(2.0));
    ShAttrib<4, SH_CONST, float> b(float(-2), float(0.2), float(0.0), float(2.0));
    ShAttrib<4, SH_CONST, ShFracUInt> expected(ShFracUInt(0), ShFracUInt(0.5), ShFracUInt(1), ShFracUInt(1));

    try {
      if (test.run(add_a_fui_4f_4f_infix_test8, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_i_3f_3f_infix_test9 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = a + b;
  } SH_END;

  add_a_i_3f_3f_infix_test9.name("add_a_i_3f_3f_infix_test9");
  last_test = "add_a_i_3f_3f_infix_test9";
  {  // i_3f_3f_infix_test9
    ShAttrib<3, SH_CONST, float> a(float(1.3), float(2.97), float(-5.4));
    ShAttrib<3, SH_CONST, float> b(float(-0.31), float(-1.5), float(8));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(1), int(2));

    try {
      if (test.run(add_a_i_3f_3f_infix_test9, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_f_4f_f_infix_test10 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<1, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = a + b;
  } SH_END;

  add_a_f_4f_f_infix_test10.name("add_a_f_4f_f_infix_test10");
  last_test = "add_a_f_4f_f_infix_test10";
  {  // f_4f_f_infix_test10
    ShAttrib<4, SH_CONST, float> a(float(3), float(4.2), float(50.0), float(-503.47));
    ShAttrib<1, SH_CONST, float> b(float(-8.2));
    ShAttrib<4, SH_CONST, float> expected(float(-5.2), float(-4.0), float(41.8), float(-511.67));

    try {
      if (test.run(add_a_f_4f_f_infix_test10, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_f_1f_3f_infix_test11 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = a + b;
  } SH_END;

  add_a_f_1f_3f_infix_test11.name("add_a_f_1f_3f_infix_test11");
  last_test = "add_a_f_1f_3f_infix_test11";
  {  // f_1f_3f_infix_test11
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<3, SH_CONST, float> b(float(3.0), float(4.0), float(5.0));
    ShAttrib<3, SH_CONST, float> expected(float(4.0), float(5.0), float(6.0));

    try {
      if (test.run(add_a_f_1f_3f_infix_test11, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_s_1s_3s_infix_test12 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, short> a;
    ShAttrib<3, SH_INPUT, short> b;
    ShAttrib<3, SH_OUTPUT, short> out;
    out = a + b;
  } SH_END;

  add_a_s_1s_3s_infix_test12.name("add_a_s_1s_3s_infix_test12");
  last_test = "add_a_s_1s_3s_infix_test12";
  {  // s_1s_3s_infix_test12
    ShAttrib<1, SH_CONST, short> a(short(1));
    ShAttrib<3, SH_CONST, short> b(short(3), short(4), short(5));
    ShAttrib<3, SH_CONST, short> expected(short(4), short(5), short(6));

    try {
      if (test.run(add_a_s_1s_3s_infix_test12, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram add_a_f_f_4f_infix_test13 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<4, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = a + b;
  } SH_END;

  add_a_f_f_4f_infix_test13.name("add_a_f_f_4f_infix_test13");
  last_test = "add_a_f_f_4f_infix_test13";
  {  // f_f_4f_infix_test13
    ShAttrib<1, SH_CONST, float> a(float(1));
    ShAttrib<4, SH_CONST, float> b(float(3), float(4.2), float(50.0), float(-503.47));
    ShAttrib<4, SH_CONST, float> expected(float(4), float(5.2), float(51.0), float(-502.47));

    try {
      if (test.run(add_a_f_f_4f_infix_test13, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_f_3f_3f_infix_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = a += b;
  } SH_END;

  madd_a_f_3f_3f_infix_test0.name("madd_a_f_3f_3f_infix_test0");
  last_test = "madd_a_f_3f_3f_infix_test0";
  {  // f_3f_3f_infix_test0
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> b(float(3.0), float(4.0), float(5.0));
    ShAttrib<3, SH_CONST, float> expected(float(3.0), float(5.0), float(7.0));

    try {
      if (test.run(madd_a_f_3f_3f_infix_test0, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_f_3f_1f_infix_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<1, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = a += b;
  } SH_END;

  madd_a_f_3f_1f_infix_test1.name("madd_a_f_3f_1f_infix_test1");
  last_test = "madd_a_f_3f_1f_infix_test1";
  {  // f_3f_1f_infix_test1
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(2.0), float(3.0));
    ShAttrib<1, SH_CONST, float> b(float(7.0));
    ShAttrib<3, SH_CONST, float> expected(float(8.0), float(9.0), float(10.0));

    try {
      if (test.run(madd_a_f_3f_1f_infix_test1, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_f_1f_1f_infix_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_INPUT, float> b;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = a += b;
  } SH_END;

  madd_a_f_1f_1f_infix_test2.name("madd_a_f_1f_1f_infix_test2");
  last_test = "madd_a_f_1f_1f_infix_test2";
  {  // f_1f_1f_infix_test2
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> b(float(7.0));
    ShAttrib<1, SH_CONST, float> expected(float(8.0));

    try {
      if (test.run(madd_a_f_1f_1f_infix_test2, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_f_3f_3f_infix_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = a += b;
  } SH_END;

  madd_a_f_3f_3f_infix_test3.name("madd_a_f_3f_3f_infix_test3");
  last_test = "madd_a_f_3f_3f_infix_test3";
  {  // f_3f_3f_infix_test3
    ShAttrib<3, SH_CONST, float> a(float(4.0), float(5.0), float(6.0));
    ShAttrib<3, SH_CONST, float> b(float(-4.0), float(-5.0), float(-6.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.0), float(0.0), float(0.0));

    try {
      if (test.run(madd_a_f_3f_3f_infix_test3, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_i_3i_3i_infix_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, int> a;
    ShAttrib<3, SH_INPUT, int> b;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = a += b;
  } SH_END;

  madd_a_i_3i_3i_infix_test4.name("madd_a_i_3i_3i_infix_test4");
  last_test = "madd_a_i_3i_3i_infix_test4";
  {  // i_3i_3i_infix_test4
    ShAttrib<3, SH_CONST, int> a(int(0), int(1), int(2));
    ShAttrib<3, SH_CONST, int> b(int(3), int(4), int(5));
    ShAttrib<3, SH_CONST, int> expected(int(3), int(5), int(7));

    try {
      if (test.run(madd_a_i_3i_3i_infix_test4, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_i_3s_1b_infix_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, short> a;
    ShAttrib<1, SH_INPUT, char> b;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = a += b;
  } SH_END;

  madd_a_i_3s_1b_infix_test5.name("madd_a_i_3s_1b_infix_test5");
  last_test = "madd_a_i_3s_1b_infix_test5";
  {  // i_3s_1b_infix_test5
    ShAttrib<3, SH_CONST, short> a(short(1), short(2), short(3));
    ShAttrib<1, SH_CONST, char> b(char(7));
    ShAttrib<3, SH_CONST, int> expected(int(8), int(9), int(10));

    try {
      if (test.run(madd_a_i_3s_1b_infix_test5, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_i_1i_1i_infix_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, int> a;
    ShAttrib<1, SH_INPUT, int> b;
    ShAttrib<1, SH_OUTPUT, int> out;
    out = a += b;
  } SH_END;

  madd_a_i_1i_1i_infix_test6.name("madd_a_i_1i_1i_infix_test6");
  last_test = "madd_a_i_1i_1i_infix_test6";
  {  // i_1i_1i_infix_test6
    ShAttrib<1, SH_CONST, int> a(int(1));
    ShAttrib<1, SH_CONST, int> b(int(7));
    ShAttrib<1, SH_CONST, int> expected(int(8));

    try {
      if (test.run(madd_a_i_1i_1i_infix_test6, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_i_3i_3i_infix_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, int> a;
    ShAttrib<3, SH_INPUT, int> b;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = a += b;
  } SH_END;

  madd_a_i_3i_3i_infix_test7.name("madd_a_i_3i_3i_infix_test7");
  last_test = "madd_a_i_3i_3i_infix_test7";
  {  // i_3i_3i_infix_test7
    ShAttrib<3, SH_CONST, int> a(int(4), int(5), int(6));
    ShAttrib<3, SH_CONST, int> b(int(-4), int(-5), int(-6));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(0), int(0));

    try {
      if (test.run(madd_a_i_3i_3i_infix_test7, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_fui_4f_4f_infix_test8 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, ShFracUInt> out;
    out = a += b;
  } SH_END;

  madd_a_fui_4f_4f_infix_test8.name("madd_a_fui_4f_4f_infix_test8");
  last_test = "madd_a_fui_4f_4f_infix_test8";
  {  // fui_4f_4f_infix_test8
    ShAttrib<4, SH_CONST, float> a(float(-4), float(0.3), float(1.0), float(2.0));
    ShAttrib<4, SH_CONST, float> b(float(-2), float(0.2), float(0.0), float(2.0));
    ShAttrib<4, SH_CONST, ShFracUInt> expected(ShFracUInt(0), ShFracUInt(0.5), ShFracUInt(1), ShFracUInt(1));

    try {
      if (test.run(madd_a_fui_4f_4f_infix_test8, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_i_3f_3f_infix_test9 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, int> out;
    out = a += b;
  } SH_END;

  madd_a_i_3f_3f_infix_test9.name("madd_a_i_3f_3f_infix_test9");
  last_test = "madd_a_i_3f_3f_infix_test9";
  {  // i_3f_3f_infix_test9
    ShAttrib<3, SH_CONST, float> a(float(1.3), float(2.97), float(-5.4));
    ShAttrib<3, SH_CONST, float> b(float(-0.31), float(-1.5), float(8));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(1), int(2));

    try {
      if (test.run(madd_a_i_3f_3f_infix_test9, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram madd_a_f_4f_f_infix_test10 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<1, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = a += b;
  } SH_END;

  madd_a_f_4f_f_infix_test10.name("madd_a_f_4f_f_infix_test10");
  last_test = "madd_a_f_4f_f_infix_test10";
  {  // f_4f_f_infix_test10
    ShAttrib<4, SH_CONST, float> a(float(3), float(4.2), float(50.0), float(-503.47));
    ShAttrib<1, SH_CONST, float> b(float(-8.2));
    ShAttrib<4, SH_CONST, float> expected(float(-5.2), float(-4.0), float(41.8), float(-511.67));

    try {
      if (test.run(madd_a_f_4f_f_infix_test10, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  {  // f_3f_3f_infix_test0
    last_test = "f_3f_3f_infix_test0";
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> b(float(3.0), float(4.0), float(5.0));
    ShAttrib<3, SH_CONST, float> expected(float(3.0), float(5.0), float(7.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = a + b;

    try {
      if (test.check("f_3f_3f_infix_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_1f_infix_test1
    last_test = "f_3f_1f_infix_test1";
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(2.0), float(3.0));
    ShAttrib<1, SH_CONST, float> b(float(7.0));
    ShAttrib<3, SH_CONST, float> expected(float(8.0), float(9.0), float(10.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = a + b;

    try {
      if (test.check("f_3f_1f_infix_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_1f_infix_test2
    last_test = "f_1f_1f_infix_test2";
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> b(float(7.0));
    ShAttrib<1, SH_CONST, float> expected(float(8.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = a + b;

    try {
      if (test.check("f_1f_1f_infix_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_3f_infix_test3
    last_test = "f_3f_3f_infix_test3";
    ShAttrib<3, SH_CONST, float> a(float(4.0), float(5.0), float(6.0));
    ShAttrib<3, SH_CONST, float> b(float(-4.0), float(-5.0), float(-6.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.0), float(0.0), float(0.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = a + b;

    try {
      if (test.check("f_3f_3f_infix_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_3i_3i_infix_test4
    last_test = "i_3i_3i_infix_test4";
    ShAttrib<3, SH_CONST, int> a(int(0), int(1), int(2));
    ShAttrib<3, SH_CONST, int> b(int(3), int(4), int(5));
    ShAttrib<3, SH_CONST, int> expected(int(3), int(5), int(7));

    ShAttrib<3, SH_TEMP, int> out;
    out = a + b;

    try {
      if (test.check("i_3i_3i_infix_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_3s_1b_infix_test5
    last_test = "i_3s_1b_infix_test5";
    ShAttrib<3, SH_CONST, short> a(short(1), short(2), short(3));
    ShAttrib<1, SH_CONST, char> b(char(7));
    ShAttrib<3, SH_CONST, int> expected(int(8), int(9), int(10));

    ShAttrib<3, SH_TEMP, int> out;
    out = a + b;

    try {
      if (test.check("i_3s_1b_infix_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_1i_1i_infix_test6
    last_test = "i_1i_1i_infix_test6";
    ShAttrib<1, SH_CONST, int> a(int(1));
    ShAttrib<1, SH_CONST, int> b(int(7));
    ShAttrib<1, SH_CONST, int> expected(int(8));

    ShAttrib<1, SH_TEMP, int> out;
    out = a + b;

    try {
      if (test.check("i_1i_1i_infix_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_3i_3i_infix_test7
    last_test = "i_3i_3i_infix_test7";
    ShAttrib<3, SH_CONST, int> a(int(4), int(5), int(6));
    ShAttrib<3, SH_CONST, int> b(int(-4), int(-5), int(-6));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(0), int(0));

    ShAttrib<3, SH_TEMP, int> out;
    out = a + b;

    try {
      if (test.check("i_3i_3i_infix_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // fui_4f_4f_infix_test8
    last_test = "fui_4f_4f_infix_test8";
    ShAttrib<4, SH_CONST, float> a(float(-4), float(0.3), float(1.0), float(2.0));
    ShAttrib<4, SH_CONST, float> b(float(-2), float(0.2), float(0.0), float(2.0));
    ShAttrib<4, SH_CONST, ShFracUInt> expected(ShFracUInt(0), ShFracUInt(0.5), ShFracUInt(1), ShFracUInt(1));

    ShAttrib<4, SH_TEMP, ShFracUInt> out;
    out = a + b;

    try {
      if (test.check("fui_4f_4f_infix_test8", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_3f_3f_infix_test9
    last_test = "i_3f_3f_infix_test9";
    ShAttrib<3, SH_CONST, float> a(float(1.3), float(2.97), float(-5.4));
    ShAttrib<3, SH_CONST, float> b(float(-0.31), float(-1.5), float(8));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(1), int(2));

    ShAttrib<3, SH_TEMP, int> out;
    out = a + b;

    try {
      if (test.check("i_3f_3f_infix_test9", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_f_infix_test10
    last_test = "f_4f_f_infix_test10";
    ShAttrib<4, SH_CONST, float> a(float(3), float(4.2), float(50.0), float(-503.47));
    ShAttrib<1, SH_CONST, float> b(float(-8.2));
    ShAttrib<4, SH_CONST, float> expected(float(-5.2), float(-4.0), float(41.8), float(-511.67));

    ShAttrib<4, SH_TEMP, float> out;
    out = a + b;

    try {
      if (test.check("f_4f_f_infix_test10", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_3f_infix_test11
    last_test = "f_1f_3f_infix_test11";
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<3, SH_CONST, float> b(float(3.0), float(4.0), float(5.0));
    ShAttrib<3, SH_CONST, float> expected(float(4.0), float(5.0), float(6.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = a + b;

    try {
      if (test.check("f_1f_3f_infix_test11", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // s_1s_3s_infix_test12
    last_test = "s_1s_3s_infix_test12";
    ShAttrib<1, SH_CONST, short> a(short(1));
    ShAttrib<3, SH_CONST, short> b(short(3), short(4), short(5));
    ShAttrib<3, SH_CONST, short> expected(short(4), short(5), short(6));

    ShAttrib<3, SH_TEMP, short> out;
    out = a + b;

    try {
      if (test.check("s_1s_3s_infix_test12", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_f_4f_infix_test13
    last_test = "f_f_4f_infix_test13";
    ShAttrib<1, SH_CONST, float> a(float(1));
    ShAttrib<4, SH_CONST, float> b(float(3), float(4.2), float(50.0), float(-503.47));
    ShAttrib<4, SH_CONST, float> expected(float(4), float(5.2), float(51.0), float(-502.47));

    ShAttrib<4, SH_TEMP, float> out;
    out = a + b;

    try {
      if (test.check("f_f_4f_infix_test13", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_3f_infix_test0
    last_test = "f_3f_3f_infix_test0";
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> b(float(3.0), float(4.0), float(5.0));
    ShAttrib<3, SH_CONST, float> expected(float(3.0), float(5.0), float(7.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = a += b;

    try {
      if (test.check("f_3f_3f_infix_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_1f_infix_test1
    last_test = "f_3f_1f_infix_test1";
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(2.0), float(3.0));
    ShAttrib<1, SH_CONST, float> b(float(7.0));
    ShAttrib<3, SH_CONST, float> expected(float(8.0), float(9.0), float(10.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = a += b;

    try {
      if (test.check("f_3f_1f_infix_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_1f_infix_test2
    last_test = "f_1f_1f_infix_test2";
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> b(float(7.0));
    ShAttrib<1, SH_CONST, float> expected(float(8.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = a += b;

    try {
      if (test.check("f_1f_1f_infix_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_3f_infix_test3
    last_test = "f_3f_3f_infix_test3";
    ShAttrib<3, SH_CONST, float> a(float(4.0), float(5.0), float(6.0));
    ShAttrib<3, SH_CONST, float> b(float(-4.0), float(-5.0), float(-6.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.0), float(0.0), float(0.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = a += b;

    try {
      if (test.check("f_3f_3f_infix_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_3i_3i_infix_test4
    last_test = "i_3i_3i_infix_test4";
    ShAttrib<3, SH_CONST, int> a(int(0), int(1), int(2));
    ShAttrib<3, SH_CONST, int> b(int(3), int(4), int(5));
    ShAttrib<3, SH_CONST, int> expected(int(3), int(5), int(7));

    ShAttrib<3, SH_TEMP, int> out;
    out = a += b;

    try {
      if (test.check("i_3i_3i_infix_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_3s_1b_infix_test5
    last_test = "i_3s_1b_infix_test5";
    ShAttrib<3, SH_CONST, short> a(short(1), short(2), short(3));
    ShAttrib<1, SH_CONST, char> b(char(7));
    ShAttrib<3, SH_CONST, int> expected(int(8), int(9), int(10));

    ShAttrib<3, SH_TEMP, int> out;
    out = a += b;

    try {
      if (test.check("i_3s_1b_infix_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_1i_1i_infix_test6
    last_test = "i_1i_1i_infix_test6";
    ShAttrib<1, SH_CONST, int> a(int(1));
    ShAttrib<1, SH_CONST, int> b(int(7));
    ShAttrib<1, SH_CONST, int> expected(int(8));

    ShAttrib<1, SH_TEMP, int> out;
    out = a += b;

    try {
      if (test.check("i_1i_1i_infix_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_3i_3i_infix_test7
    last_test = "i_3i_3i_infix_test7";
    ShAttrib<3, SH_CONST, int> a(int(4), int(5), int(6));
    ShAttrib<3, SH_CONST, int> b(int(-4), int(-5), int(-6));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(0), int(0));

    ShAttrib<3, SH_TEMP, int> out;
    out = a += b;

    try {
      if (test.check("i_3i_3i_infix_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // fui_4f_4f_infix_test8
    last_test = "fui_4f_4f_infix_test8";
    ShAttrib<4, SH_CONST, float> a(float(-4), float(0.3), float(1.0), float(2.0));
    ShAttrib<4, SH_CONST, float> b(float(-2), float(0.2), float(0.0), float(2.0));
    ShAttrib<4, SH_CONST, ShFracUInt> expected(ShFracUInt(0), ShFracUInt(0.5), ShFracUInt(1), ShFracUInt(1));

    ShAttrib<4, SH_TEMP, ShFracUInt> out;
    out = a += b;

    try {
      if (test.check("fui_4f_4f_infix_test8", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // i_3f_3f_infix_test9
    last_test = "i_3f_3f_infix_test9";
    ShAttrib<3, SH_CONST, float> a(float(1.3), float(2.97), float(-5.4));
    ShAttrib<3, SH_CONST, float> b(float(-0.31), float(-1.5), float(8));
    ShAttrib<3, SH_CONST, int> expected(int(0), int(1), int(2));

    ShAttrib<3, SH_TEMP, int> out;
    out = a += b;

    try {
      if (test.check("i_3f_3f_infix_test9", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_f_infix_test10
    last_test = "f_4f_f_infix_test10";
    ShAttrib<4, SH_CONST, float> a(float(3), float(4.2), float(50.0), float(-503.47));
    ShAttrib<1, SH_CONST, float> b(float(-8.2));
    ShAttrib<4, SH_CONST, float> expected(float(-5.2), float(-4.0), float(41.8), float(-511.67));

    ShAttrib<4, SH_TEMP, float> out;
    out = a += b;

    try {
      if (test.check("f_4f_f_infix_test10", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  if (errors !=0) {
    std::cout << "Total Errors: " << errors << "/" << total_tests << std::endl;
    return 1;
  }
}

