#include "test.hpp"

int main(int argc, char** argv) {
  using namespace SH;

  char* last_test = "none";

  Test test(argc, argv);

  int errors = 0, total_tests = 0;
  ShProgram cbrt_a_f_4f_callcbrt_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = cbrt(a);
  } SH_END;

  cbrt_a_f_4f_callcbrt_test0.name("cbrt_a_f_4f_callcbrt_test0");
  last_test = "cbrt_a_f_4f_callcbrt_test0";
  {  // f_4f_callcbrt_test0
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(1.0), float(1.25992104989), float(1.44224957031));

    try {
      if (test.run(cbrt_a_f_4f_callcbrt_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cbrt_a_f_1f_callcbrt_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = cbrt(a);
  } SH_END;

  cbrt_a_f_1f_callcbrt_test1.name("cbrt_a_f_1f_callcbrt_test1");
  last_test = "cbrt_a_f_1f_callcbrt_test1";
  {  // f_1f_callcbrt_test1
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> expected(float(1.0));

    try {
      if (test.run(cbrt_a_f_1f_callcbrt_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cbrt_a_f_2f_callcbrt_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = cbrt(a);
  } SH_END;

  cbrt_a_f_2f_callcbrt_test2.name("cbrt_a_f_2f_callcbrt_test2");
  last_test = "cbrt_a_f_2f_callcbrt_test2";
  {  // f_2f_callcbrt_test2
    ShAttrib<2, SH_CONST, float> a(float(4000.2), float(27));
    ShAttrib<2, SH_CONST, float> expected(float(15.8742750821), float(3.0));

    try {
      if (test.run(cbrt_a_f_2f_callcbrt_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  {  // f_4f_callcbrt_test0
    last_test = "f_4f_callcbrt_test0";
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(1.0), float(1.25992104989), float(1.44224957031));

    ShAttrib<4, SH_TEMP, float> out;
    out = cbrt(a);

    try {
      if (test.check("f_4f_callcbrt_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callcbrt_test1
    last_test = "f_1f_callcbrt_test1";
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> expected(float(1.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = cbrt(a);

    try {
      if (test.check("f_1f_callcbrt_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callcbrt_test2
    last_test = "f_2f_callcbrt_test2";
    ShAttrib<2, SH_CONST, float> a(float(4000.2), float(27));
    ShAttrib<2, SH_CONST, float> expected(float(15.8742750821), float(3.0));

    ShAttrib<2, SH_TEMP, float> out;
    out = cbrt(a);

    try {
      if (test.check("f_2f_callcbrt_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  if (errors !=0) {
    std::cout << "Total Errors: " << errors << "/" << total_tests << std::endl;
    return 1;
  }
}

