#include "test.hpp"

int main(int argc, char** argv) {
  using namespace SH;

  char* last_test = "none";

  Test test(argc, argv);

  int errors = 0, total_tests = 0;
  ShProgram exp_a_f_3f_callexp_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = exp(a);
  } SH_END;

  exp_a_f_3f_callexp_test0.name("exp_a_f_3f_callexp_test0");
  last_test = "exp_a_f_3f_callexp_test0";
  {  // f_3f_callexp_test0
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(2.71828182846), float(7.38905609893));

    try {
      if (test.run(exp_a_f_3f_callexp_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp_a_f_4f_callexp_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = exp(a);
  } SH_END;

  exp_a_f_4f_callexp_test1.name("exp_a_f_4f_callexp_test1");
  last_test = "exp_a_f_4f_callexp_test1";
  {  // f_4f_callexp_test1
    ShAttrib<4, SH_CONST, float> a(float(0.1), float(0.25), float(0.3), float(0.5));
    ShAttrib<4, SH_CONST, float> expected(float(1.10517091808), float(1.28402541669), float(1.34985880758), float(1.6487212707));

    try {
      if (test.run(exp_a_f_4f_callexp_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp_a_f_2f_callexp_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp(a);
  } SH_END;

  exp_a_f_2f_callexp_test2.name("exp_a_f_2f_callexp_test2");
  last_test = "exp_a_f_2f_callexp_test2";
  {  // f_2f_callexp_test2
    ShAttrib<2, SH_CONST, float> a(float(-2.0), float(-3.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.135335283237), float(0.0497870683679));

    try {
      if (test.run(exp_a_f_2f_callexp_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp_a_f_2f_callexp_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp(a);
  } SH_END;

  exp_a_f_2f_callexp_test3.name("exp_a_f_2f_callexp_test3");
  last_test = "exp_a_f_2f_callexp_test3";
  {  // f_2f_callexp_test3
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-1.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.606530659713), float(0.367879441171));

    try {
      if (test.run(exp_a_f_2f_callexp_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp_a_f_2f_callexp_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp(a);
  } SH_END;

  exp_a_f_2f_callexp_test4.name("exp_a_f_2f_callexp_test4");
  last_test = "exp_a_f_2f_callexp_test4";
  {  // f_2f_callexp_test4
    ShAttrib<2, SH_CONST, float> a(float(2.3), float(2.9));
    ShAttrib<2, SH_CONST, float> expected(float(9.97418245481), float(18.1741453694));

    try {
      if (test.run(exp_a_f_2f_callexp_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp_a_f_2f_callexp_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp(a);
  } SH_END;

  exp_a_f_2f_callexp_test5.name("exp_a_f_2f_callexp_test5");
  last_test = "exp_a_f_2f_callexp_test5";
  {  // f_2f_callexp_test5
    ShAttrib<2, SH_CONST, float> a(float(3.8), float(4.0));
    ShAttrib<2, SH_CONST, float> expected(float(44.7011844933), float(54.5981500331));

    try {
      if (test.run(exp_a_f_2f_callexp_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp2_a_f_3f_callexp2_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = exp2(a);
  } SH_END;

  exp2_a_f_3f_callexp2_test0.name("exp2_a_f_3f_callexp2_test0");
  last_test = "exp2_a_f_3f_callexp2_test0";
  {  // f_3f_callexp2_test0
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(2.0), float(4.0));

    try {
      if (test.run(exp2_a_f_3f_callexp2_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp2_a_f_4f_callexp2_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = exp2(a);
  } SH_END;

  exp2_a_f_4f_callexp2_test1.name("exp2_a_f_4f_callexp2_test1");
  last_test = "exp2_a_f_4f_callexp2_test1";
  {  // f_4f_callexp2_test1
    ShAttrib<4, SH_CONST, float> a(float(0.1), float(0.25), float(0.3), float(0.5));
    ShAttrib<4, SH_CONST, float> expected(float(1.07177346254), float(1.189207115), float(1.23114441334), float(1.41421356237));

    try {
      if (test.run(exp2_a_f_4f_callexp2_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp2_a_f_2f_callexp2_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp2(a);
  } SH_END;

  exp2_a_f_2f_callexp2_test2.name("exp2_a_f_2f_callexp2_test2");
  last_test = "exp2_a_f_2f_callexp2_test2";
  {  // f_2f_callexp2_test2
    ShAttrib<2, SH_CONST, float> a(float(-2.0), float(-3.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.25), float(0.125));

    try {
      if (test.run(exp2_a_f_2f_callexp2_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp2_a_f_2f_callexp2_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp2(a);
  } SH_END;

  exp2_a_f_2f_callexp2_test3.name("exp2_a_f_2f_callexp2_test3");
  last_test = "exp2_a_f_2f_callexp2_test3";
  {  // f_2f_callexp2_test3
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-1.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.707106781187), float(0.5));

    try {
      if (test.run(exp2_a_f_2f_callexp2_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp2_a_f_2f_callexp2_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp2(a);
  } SH_END;

  exp2_a_f_2f_callexp2_test4.name("exp2_a_f_2f_callexp2_test4");
  last_test = "exp2_a_f_2f_callexp2_test4";
  {  // f_2f_callexp2_test4
    ShAttrib<2, SH_CONST, float> a(float(2.3), float(2.9));
    ShAttrib<2, SH_CONST, float> expected(float(4.92457765338), float(7.46426393229));

    try {
      if (test.run(exp2_a_f_2f_callexp2_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp2_a_f_2f_callexp2_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp2(a);
  } SH_END;

  exp2_a_f_2f_callexp2_test5.name("exp2_a_f_2f_callexp2_test5");
  last_test = "exp2_a_f_2f_callexp2_test5";
  {  // f_2f_callexp2_test5
    ShAttrib<2, SH_CONST, float> a(float(3.8), float(4.0));
    ShAttrib<2, SH_CONST, float> expected(float(13.9288090127), float(16.0));

    try {
      if (test.run(exp2_a_f_2f_callexp2_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp10_a_f_3f_callexp10_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = exp10(a);
  } SH_END;

  exp10_a_f_3f_callexp10_test0.name("exp10_a_f_3f_callexp10_test0");
  last_test = "exp10_a_f_3f_callexp10_test0";
  {  // f_3f_callexp10_test0
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(10.0), float(100.0));

    try {
      if (test.run(exp10_a_f_3f_callexp10_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp10_a_f_4f_callexp10_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = exp10(a);
  } SH_END;

  exp10_a_f_4f_callexp10_test1.name("exp10_a_f_4f_callexp10_test1");
  last_test = "exp10_a_f_4f_callexp10_test1";
  {  // f_4f_callexp10_test1
    ShAttrib<4, SH_CONST, float> a(float(0.1), float(0.25), float(0.3), float(0.5));
    ShAttrib<4, SH_CONST, float> expected(float(1.25892541179), float(1.77827941004), float(1.99526231497), float(3.16227766017));

    try {
      if (test.run(exp10_a_f_4f_callexp10_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp10_a_f_2f_callexp10_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp10(a);
  } SH_END;

  exp10_a_f_2f_callexp10_test2.name("exp10_a_f_2f_callexp10_test2");
  last_test = "exp10_a_f_2f_callexp10_test2";
  {  // f_2f_callexp10_test2
    ShAttrib<2, SH_CONST, float> a(float(-2.0), float(-3.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.01), float(0.001));

    try {
      if (test.run(exp10_a_f_2f_callexp10_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp10_a_f_2f_callexp10_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp10(a);
  } SH_END;

  exp10_a_f_2f_callexp10_test3.name("exp10_a_f_2f_callexp10_test3");
  last_test = "exp10_a_f_2f_callexp10_test3";
  {  // f_2f_callexp10_test3
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-1.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.316227766017), float(0.1));

    try {
      if (test.run(exp10_a_f_2f_callexp10_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp10_a_f_2f_callexp10_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp10(a);
  } SH_END;

  exp10_a_f_2f_callexp10_test4.name("exp10_a_f_2f_callexp10_test4");
  last_test = "exp10_a_f_2f_callexp10_test4";
  {  // f_2f_callexp10_test4
    ShAttrib<2, SH_CONST, float> a(float(2.3), float(2.9));
    ShAttrib<2, SH_CONST, float> expected(float(199.526231497), float(794.328234724));

    try {
      if (test.run(exp10_a_f_2f_callexp10_test4, a, expected, 0.1) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram exp10_a_f_2f_callexp10_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = exp10(a);
  } SH_END;

  exp10_a_f_2f_callexp10_test5.name("exp10_a_f_2f_callexp10_test5");
  last_test = "exp10_a_f_2f_callexp10_test5";
  {  // f_2f_callexp10_test5
    ShAttrib<2, SH_CONST, float> a(float(3.8), float(4.0));
    ShAttrib<2, SH_CONST, float> expected(float(6309.5734448), float(10000.0));

    try {
      if (test.run(exp10_a_f_2f_callexp10_test5, a, expected, 1) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  {  // f_3f_callexp_test0
    last_test = "f_3f_callexp_test0";
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(2.71828182846), float(7.38905609893));

    ShAttrib<3, SH_TEMP, float> out;
    out = exp(a);

    try {
      if (test.check("f_3f_callexp_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callexp_test1
    last_test = "f_4f_callexp_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.1), float(0.25), float(0.3), float(0.5));
    ShAttrib<4, SH_CONST, float> expected(float(1.10517091808), float(1.28402541669), float(1.34985880758), float(1.6487212707));

    ShAttrib<4, SH_TEMP, float> out;
    out = exp(a);

    try {
      if (test.check("f_4f_callexp_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp_test2
    last_test = "f_2f_callexp_test2";
    ShAttrib<2, SH_CONST, float> a(float(-2.0), float(-3.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.135335283237), float(0.0497870683679));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp(a);

    try {
      if (test.check("f_2f_callexp_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp_test3
    last_test = "f_2f_callexp_test3";
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-1.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.606530659713), float(0.367879441171));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp(a);

    try {
      if (test.check("f_2f_callexp_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp_test4
    last_test = "f_2f_callexp_test4";
    ShAttrib<2, SH_CONST, float> a(float(2.3), float(2.9));
    ShAttrib<2, SH_CONST, float> expected(float(9.97418245481), float(18.1741453694));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp(a);

    try {
      if (test.check("f_2f_callexp_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp_test5
    last_test = "f_2f_callexp_test5";
    ShAttrib<2, SH_CONST, float> a(float(3.8), float(4.0));
    ShAttrib<2, SH_CONST, float> expected(float(44.7011844933), float(54.5981500331));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp(a);

    try {
      if (test.check("f_2f_callexp_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callexp2_test0
    last_test = "f_3f_callexp2_test0";
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(2.0), float(4.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = exp2(a);

    try {
      if (test.check("f_3f_callexp2_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callexp2_test1
    last_test = "f_4f_callexp2_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.1), float(0.25), float(0.3), float(0.5));
    ShAttrib<4, SH_CONST, float> expected(float(1.07177346254), float(1.189207115), float(1.23114441334), float(1.41421356237));

    ShAttrib<4, SH_TEMP, float> out;
    out = exp2(a);

    try {
      if (test.check("f_4f_callexp2_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp2_test2
    last_test = "f_2f_callexp2_test2";
    ShAttrib<2, SH_CONST, float> a(float(-2.0), float(-3.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.25), float(0.125));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp2(a);

    try {
      if (test.check("f_2f_callexp2_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp2_test3
    last_test = "f_2f_callexp2_test3";
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-1.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.707106781187), float(0.5));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp2(a);

    try {
      if (test.check("f_2f_callexp2_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp2_test4
    last_test = "f_2f_callexp2_test4";
    ShAttrib<2, SH_CONST, float> a(float(2.3), float(2.9));
    ShAttrib<2, SH_CONST, float> expected(float(4.92457765338), float(7.46426393229));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp2(a);

    try {
      if (test.check("f_2f_callexp2_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp2_test5
    last_test = "f_2f_callexp2_test5";
    ShAttrib<2, SH_CONST, float> a(float(3.8), float(4.0));
    ShAttrib<2, SH_CONST, float> expected(float(13.9288090127), float(16.0));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp2(a);

    try {
      if (test.check("f_2f_callexp2_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callexp10_test0
    last_test = "f_3f_callexp10_test0";
    ShAttrib<3, SH_CONST, float> a(float(0.0), float(1.0), float(2.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.0), float(10.0), float(100.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = exp10(a);

    try {
      if (test.check("f_3f_callexp10_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callexp10_test1
    last_test = "f_4f_callexp10_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.1), float(0.25), float(0.3), float(0.5));
    ShAttrib<4, SH_CONST, float> expected(float(1.25892541179), float(1.77827941004), float(1.99526231497), float(3.16227766017));

    ShAttrib<4, SH_TEMP, float> out;
    out = exp10(a);

    try {
      if (test.check("f_4f_callexp10_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp10_test2
    last_test = "f_2f_callexp10_test2";
    ShAttrib<2, SH_CONST, float> a(float(-2.0), float(-3.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.01), float(0.001));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp10(a);

    try {
      if (test.check("f_2f_callexp10_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp10_test3
    last_test = "f_2f_callexp10_test3";
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-1.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.316227766017), float(0.1));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp10(a);

    try {
      if (test.check("f_2f_callexp10_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp10_test4
    last_test = "f_2f_callexp10_test4";
    ShAttrib<2, SH_CONST, float> a(float(2.3), float(2.9));
    ShAttrib<2, SH_CONST, float> expected(float(199.526231497), float(794.328234724));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp10(a);

    try {
      if (test.check("f_2f_callexp10_test4", out, expected, 0.1) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callexp10_test5
    last_test = "f_2f_callexp10_test5";
    ShAttrib<2, SH_CONST, float> a(float(3.8), float(4.0));
    ShAttrib<2, SH_CONST, float> expected(float(6309.5734448), float(10000.0));

    ShAttrib<2, SH_TEMP, float> out;
    out = exp10(a);

    try {
      if (test.check("f_2f_callexp10_test5", out, expected, 1) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  if (errors !=0) {
    std::cout << "Total Errors: " << errors << "/" << total_tests << std::endl;
    return 1;
  }
}

