#include "test.hpp"

int main(int argc, char** argv) {
  using namespace SH;

  char* last_test = "none";

  Test test(argc, argv);

  int errors = 0, total_tests = 0;
  ShProgram cos_a_f_4f_callcos_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = cos(a);
  } SH_END;

  cos_a_f_4f_callcos_test0.name("cos_a_f_4f_callcos_test0");
  last_test = "cos_a_f_4f_callcos_test0";
  {  // f_4f_callcos_test0
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(1.0), float(0.540302305868), float(-0.416146836547), float(-0.9899924966));

    try {
      if (test.run(cos_a_f_4f_callcos_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cos_a_f_4f_callcos_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = cos(a);
  } SH_END;

  cos_a_f_4f_callcos_test1.name("cos_a_f_4f_callcos_test1");
  last_test = "cos_a_f_4f_callcos_test1";
  {  // f_4f_callcos_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.955336489126), float(0.87758256189), float(0.696706709347), float(0.621609968271));

    try {
      if (test.run(cos_a_f_4f_callcos_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cos_a_f_3f_callcos_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = cos(a);
  } SH_END;

  cos_a_f_3f_callcos_test2.name("cos_a_f_3f_callcos_test2");
  last_test = "cos_a_f_3f_callcos_test2";
  {  // f_3f_callcos_test2
    ShAttrib<3, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(-1.0), float(-1.0), float(1.0));

    try {
      if (test.run(cos_a_f_3f_callcos_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cos_a_f_2f_callcos_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = cos(a);
  } SH_END;

  cos_a_f_2f_callcos_test3.name("cos_a_f_2f_callcos_test3");
  last_test = "cos_a_f_2f_callcos_test3";
  {  // f_2f_callcos_test3
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(-6.28318530718));
    ShAttrib<2, SH_CONST, float> expected(float(1.0), float(1.0));

    try {
      if (test.run(cos_a_f_2f_callcos_test3, a, expected, 0.06) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cos_a_f_2f_callcos_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = cos(a);
  } SH_END;

  cos_a_f_2f_callcos_test4.name("cos_a_f_2f_callcos_test4");
  last_test = "cos_a_f_2f_callcos_test4";
  {  // f_2f_callcos_test4
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(6.12303176911e-17), float(6.12303176911e-17));

    try {
      if (test.run(cos_a_f_2f_callcos_test4, a, expected, 0.0005) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cos_a_f_1f_callcos_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = cos(a);
  } SH_END;

  cos_a_f_1f_callcos_test5.name("cos_a_f_1f_callcos_test5");
  last_test = "cos_a_f_1f_callcos_test5";
  {  // f_1f_callcos_test5
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(-0.9899924966));

    try {
      if (test.run(cos_a_f_1f_callcos_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cos_a_f_4f_callcos_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = cos(a);
  } SH_END;

  cos_a_f_4f_callcos_test6.name("cos_a_f_4f_callcos_test6");
  last_test = "cos_a_f_4f_callcos_test6";
  {  // f_4f_callcos_test6
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.87758256189), float(0.540302305868), float(-0.9899924966), float(-0.653643620864));

    try {
      if (test.run(cos_a_f_4f_callcos_test6, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cos_a_f_3f_callcos_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = cos(a);
  } SH_END;

  cos_a_f_3f_callcos_test7.name("cos_a_f_3f_callcos_test7");
  last_test = "cos_a_f_3f_callcos_test7";
  {  // f_3f_callcos_test7
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.87758256189), float(0.0707372016677), float(-0.801143615547));

    try {
      if (test.run(cos_a_f_3f_callcos_test7, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acos_a_f_1f_callacos_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = acos(a);
  } SH_END;

  acos_a_f_1f_callacos_test0.name("acos_a_f_1f_callacos_test0");
  last_test = "acos_a_f_1f_callacos_test0";
  {  // f_1f_callacos_test0
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(1.57079632679));

    try {
      if (test.run(acos_a_f_1f_callacos_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acos_a_f_4f_callacos_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = acos(a);
  } SH_END;

  acos_a_f_4f_callacos_test1.name("acos_a_f_4f_callacos_test1");
  last_test = "acos_a_f_4f_callacos_test1";
  {  // f_4f_callacos_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(1.26610367278), float(1.0471975512), float(0.643501108793), float(0.451026811796));

    try {
      if (test.run(acos_a_f_4f_callacos_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acos_a_f_2f_callacos_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = acos(a);
  } SH_END;

  acos_a_f_2f_callacos_test2.name("acos_a_f_2f_callacos_test2");
  last_test = "acos_a_f_2f_callacos_test2";
  {  // f_2f_callacos_test2
    ShAttrib<2, SH_CONST, float> a(float(0.9), float(0.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.451026811796), float(1.57079632679));

    try {
      if (test.run(acos_a_f_2f_callacos_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acos_a_f_1f_callacos_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = acos(a);
  } SH_END;

  acos_a_f_1f_callacos_test3.name("acos_a_f_1f_callacos_test3");
  last_test = "acos_a_f_1f_callacos_test3";
  {  // f_1f_callacos_test3
    ShAttrib<1, SH_CONST, float> a(float(-0.9));
    ShAttrib<1, SH_CONST, float> expected(float(2.69056584179));

    try {
      if (test.run(acos_a_f_1f_callacos_test3, a, expected, 0.09) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acos_a_f_2f_callacos_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = acos(a);
  } SH_END;

  acos_a_f_2f_callacos_test4.name("acos_a_f_2f_callacos_test4");
  last_test = "acos_a_f_2f_callacos_test4";
  {  // f_2f_callacos_test4
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-0.1));
    ShAttrib<2, SH_CONST, float> expected(float(2.09439510239), float(1.67096374796));

    try {
      if (test.run(acos_a_f_2f_callacos_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acos_a_f_3f_callacos_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = acos(a);
  } SH_END;

  acos_a_f_3f_callacos_test5.name("acos_a_f_3f_callacos_test5");
  last_test = "acos_a_f_3f_callacos_test5";
  {  // f_3f_callacos_test5
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(0.6), float(0.9));
    ShAttrib<3, SH_CONST, float> expected(float(1.0471975512), float(0.927295218002), float(0.451026811796));

    try {
      if (test.run(acos_a_f_3f_callacos_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sin_a_f_4f_callsin_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = sin(a);
  } SH_END;

  sin_a_f_4f_callsin_test0.name("sin_a_f_4f_callsin_test0");
  last_test = "sin_a_f_4f_callsin_test0";
  {  // f_4f_callsin_test0
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(0.841470984808), float(0.909297426826), float(0.14112000806));

    try {
      if (test.run(sin_a_f_4f_callsin_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sin_a_f_4f_callsin_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = sin(a);
  } SH_END;

  sin_a_f_4f_callsin_test1.name("sin_a_f_4f_callsin_test1");
  last_test = "sin_a_f_4f_callsin_test1";
  {  // f_4f_callsin_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.295520206661), float(0.479425538604), float(0.7173560909), float(0.783326909627));

    try {
      if (test.run(sin_a_f_4f_callsin_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sin_a_f_3f_callsin_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = sin(a);
  } SH_END;

  sin_a_f_3f_callsin_test2.name("sin_a_f_3f_callsin_test2");
  last_test = "sin_a_f_3f_callsin_test2";
  {  // f_3f_callsin_test2
    ShAttrib<3, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.22460635382e-16), float(-1.22460635382e-16), float(0.0));

    try {
      if (test.run(sin_a_f_3f_callsin_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sin_a_f_2f_callsin_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = sin(a);
  } SH_END;

  sin_a_f_2f_callsin_test3.name("sin_a_f_2f_callsin_test3");
  last_test = "sin_a_f_2f_callsin_test3";
  {  // f_2f_callsin_test3
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(-6.28318530718));
    ShAttrib<2, SH_CONST, float> expected(float(-2.44921270764e-16), float(2.44921270764e-16));

    try {
      if (test.run(sin_a_f_2f_callsin_test3, a, expected, 0.06) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sin_a_f_2f_callsin_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = sin(a);
  } SH_END;

  sin_a_f_2f_callsin_test4.name("sin_a_f_2f_callsin_test4");
  last_test = "sin_a_f_2f_callsin_test4";
  {  // f_2f_callsin_test4
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(1.0), float(-1.0));

    try {
      if (test.run(sin_a_f_2f_callsin_test4, a, expected, 0.0005) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sin_a_f_1f_callsin_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = sin(a);
  } SH_END;

  sin_a_f_1f_callsin_test5.name("sin_a_f_1f_callsin_test5");
  last_test = "sin_a_f_1f_callsin_test5";
  {  // f_1f_callsin_test5
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.14112000806));

    try {
      if (test.run(sin_a_f_1f_callsin_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sin_a_f_4f_callsin_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = sin(a);
  } SH_END;

  sin_a_f_4f_callsin_test6.name("sin_a_f_4f_callsin_test6");
  last_test = "sin_a_f_4f_callsin_test6";
  {  // f_4f_callsin_test6
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(-0.479425538604), float(-0.841470984808), float(-0.14112000806), float(0.756802495308));

    try {
      if (test.run(sin_a_f_4f_callsin_test6, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sin_a_f_3f_callsin_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = sin(a);
  } SH_END;

  sin_a_f_3f_callsin_test7.name("sin_a_f_3f_callsin_test7");
  last_test = "sin_a_f_3f_callsin_test7";
  {  // f_3f_callsin_test7
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.479425538604), float(0.997494986604), float(0.598472144104));

    try {
      if (test.run(sin_a_f_3f_callsin_test7, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asin_a_f_1f_callasin_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = asin(a);
  } SH_END;

  asin_a_f_1f_callasin_test0.name("asin_a_f_1f_callasin_test0");
  last_test = "asin_a_f_1f_callasin_test0";
  {  // f_1f_callasin_test0
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    try {
      if (test.run(asin_a_f_1f_callasin_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asin_a_f_4f_callasin_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = asin(a);
  } SH_END;

  asin_a_f_4f_callasin_test1.name("asin_a_f_4f_callasin_test1");
  last_test = "asin_a_f_4f_callasin_test1";
  {  // f_4f_callasin_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.304692654015), float(0.523598775598), float(0.927295218002), float(1.119769515));

    try {
      if (test.run(asin_a_f_4f_callasin_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asin_a_f_2f_callasin_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = asin(a);
  } SH_END;

  asin_a_f_2f_callasin_test2.name("asin_a_f_2f_callasin_test2");
  last_test = "asin_a_f_2f_callasin_test2";
  {  // f_2f_callasin_test2
    ShAttrib<2, SH_CONST, float> a(float(0.9), float(0.0));
    ShAttrib<2, SH_CONST, float> expected(float(1.119769515), float(0.0));

    try {
      if (test.run(asin_a_f_2f_callasin_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asin_a_f_1f_callasin_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = asin(a);
  } SH_END;

  asin_a_f_1f_callasin_test3.name("asin_a_f_1f_callasin_test3");
  last_test = "asin_a_f_1f_callasin_test3";
  {  // f_1f_callasin_test3
    ShAttrib<1, SH_CONST, float> a(float(-0.9));
    ShAttrib<1, SH_CONST, float> expected(float(-1.119769515));

    try {
      if (test.run(asin_a_f_1f_callasin_test3, a, expected, 0.09) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asin_a_f_2f_callasin_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = asin(a);
  } SH_END;

  asin_a_f_2f_callasin_test4.name("asin_a_f_2f_callasin_test4");
  last_test = "asin_a_f_2f_callasin_test4";
  {  // f_2f_callasin_test4
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-0.1));
    ShAttrib<2, SH_CONST, float> expected(float(-0.523598775598), float(-0.100167421162));

    try {
      if (test.run(asin_a_f_2f_callasin_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asin_a_f_3f_callasin_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = asin(a);
  } SH_END;

  asin_a_f_3f_callasin_test5.name("asin_a_f_3f_callasin_test5");
  last_test = "asin_a_f_3f_callasin_test5";
  {  // f_3f_callasin_test5
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(0.6), float(0.9));
    ShAttrib<3, SH_CONST, float> expected(float(0.523598775598), float(0.643501108793), float(1.119769515));

    try {
      if (test.run(asin_a_f_3f_callasin_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tan_a_f_4f_calltan_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = tan(a);
  } SH_END;

  tan_a_f_4f_calltan_test0.name("tan_a_f_4f_calltan_test0");
  last_test = "tan_a_f_4f_calltan_test0";
  {  // f_4f_calltan_test0
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(1.55740772465), float(-2.18503986326), float(-0.142546543074));

    try {
      if (test.run(tan_a_f_4f_calltan_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tan_a_f_4f_calltan_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = tan(a);
  } SH_END;

  tan_a_f_4f_calltan_test1.name("tan_a_f_4f_calltan_test1");
  last_test = "tan_a_f_4f_calltan_test1";
  {  // f_4f_calltan_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.30933624961), float(0.546302489844), float(1.02963855705), float(1.26015821755));

    try {
      if (test.run(tan_a_f_4f_calltan_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tan_a_f_3f_calltan_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = tan(a);
  } SH_END;

  tan_a_f_3f_calltan_test2.name("tan_a_f_3f_calltan_test2");
  last_test = "tan_a_f_3f_calltan_test2";
  {  // f_3f_calltan_test2
    ShAttrib<3, SH_CONST, float> a(float(1.47079632679), float(-1.47079632679), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(9.96664442326), float(-9.96664442326), float(0.0));

    try {
      if (test.run(tan_a_f_3f_calltan_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tan_a_f_3f_calltan_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = tan(a);
  } SH_END;

  tan_a_f_3f_calltan_test3.name("tan_a_f_3f_calltan_test3");
  last_test = "tan_a_f_3f_calltan_test3";
  {  // f_3f_calltan_test3
    ShAttrib<3, SH_CONST, float> a(float(-0.5), float(-1.0), float(-1.1));
    ShAttrib<3, SH_CONST, float> expected(float(-0.546302489844), float(-1.55740772465), float(-1.96475965725));

    try {
      if (test.run(tan_a_f_3f_calltan_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tan_a_f_3f_calltan_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = tan(a);
  } SH_END;

  tan_a_f_3f_calltan_test4.name("tan_a_f_3f_calltan_test4");
  last_test = "tan_a_f_3f_calltan_test4";
  {  // f_3f_calltan_test4
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.546302489844), float(14.1014199472), float(-0.747022297239));

    try {
      if (test.run(tan_a_f_3f_calltan_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cosh_a_f_4f_callcosh_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = cosh(a);
  } SH_END;

  cosh_a_f_4f_callcosh_test0.name("cosh_a_f_4f_callcosh_test0");
  last_test = "cosh_a_f_4f_callcosh_test0";
  {  // f_4f_callcosh_test0
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(1.0), float(1.54308063482), float(3.76219569108), float(10.0676619958));

    try {
      if (test.run(cosh_a_f_4f_callcosh_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cosh_a_f_4f_callcosh_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = cosh(a);
  } SH_END;

  cosh_a_f_4f_callcosh_test1.name("cosh_a_f_4f_callcosh_test1");
  last_test = "cosh_a_f_4f_callcosh_test1";
  {  // f_4f_callcosh_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(1.04533851413), float(1.12762596521), float(1.3374349463), float(1.43308638545));

    try {
      if (test.run(cosh_a_f_4f_callcosh_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cosh_a_f_3f_callcosh_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = cosh(a);
  } SH_END;

  cosh_a_f_3f_callcosh_test2.name("cosh_a_f_3f_callcosh_test2");
  last_test = "cosh_a_f_3f_callcosh_test2";
  {  // f_3f_callcosh_test2
    ShAttrib<3, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(11.5919532755), float(11.5919532755), float(1.0));

    try {
      if (test.run(cosh_a_f_3f_callcosh_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cosh_a_f_2f_callcosh_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = cosh(a);
  } SH_END;

  cosh_a_f_2f_callcosh_test3.name("cosh_a_f_2f_callcosh_test3");
  last_test = "cosh_a_f_2f_callcosh_test3";
  {  // f_2f_callcosh_test3
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(-6.28318530718));
    ShAttrib<2, SH_CONST, float> expected(float(267.746761484), float(267.746761484));

    try {
      if (test.run(cosh_a_f_2f_callcosh_test3, a, expected, 0.06) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cosh_a_f_2f_callcosh_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = cosh(a);
  } SH_END;

  cosh_a_f_2f_callcosh_test4.name("cosh_a_f_2f_callcosh_test4");
  last_test = "cosh_a_f_2f_callcosh_test4";
  {  // f_2f_callcosh_test4
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(2.50917847866), float(2.50917847866));

    try {
      if (test.run(cosh_a_f_2f_callcosh_test4, a, expected, 0.0005) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cosh_a_f_1f_callcosh_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = cosh(a);
  } SH_END;

  cosh_a_f_1f_callcosh_test5.name("cosh_a_f_1f_callcosh_test5");
  last_test = "cosh_a_f_1f_callcosh_test5";
  {  // f_1f_callcosh_test5
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(10.0676619958));

    try {
      if (test.run(cosh_a_f_1f_callcosh_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cosh_a_f_4f_callcosh_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = cosh(a);
  } SH_END;

  cosh_a_f_4f_callcosh_test6.name("cosh_a_f_4f_callcosh_test6");
  last_test = "cosh_a_f_4f_callcosh_test6";
  {  // f_4f_callcosh_test6
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(1.12762596521), float(1.54308063482), float(10.0676619958), float(27.308232836));

    try {
      if (test.run(cosh_a_f_4f_callcosh_test6, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram cosh_a_f_3f_callcosh_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = cosh(a);
  } SH_END;

  cosh_a_f_3f_callcosh_test7.name("cosh_a_f_3f_callcosh_test7");
  last_test = "cosh_a_f_3f_callcosh_test7";
  {  // f_3f_callcosh_test7
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(1.12762596521), float(2.35240961524), float(6.13228947966));

    try {
      if (test.run(cosh_a_f_3f_callcosh_test7, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acosh_a_f_1f_callacosh_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = acosh(a);
  } SH_END;

  acosh_a_f_1f_callacosh_test0.name("acosh_a_f_1f_callacosh_test0");
  last_test = "acosh_a_f_1f_callacosh_test0";
  {  // f_1f_callacosh_test0
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> expected(float(4.4408920985e-16));

    try {
      if (test.run(acosh_a_f_1f_callacosh_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acosh_a_f_2f_callacosh_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = acosh(a);
  } SH_END;

  acosh_a_f_2f_callacosh_test1.name("acosh_a_f_2f_callacosh_test1");
  last_test = "acosh_a_f_2f_callacosh_test1";
  {  // f_2f_callacosh_test1
    ShAttrib<2, SH_CONST, float> a(float(50.3), float(2000.5));
    ShAttrib<2, SH_CONST, float> expected(float(4.6110534323), float(8.29429954639));

    try {
      if (test.run(acosh_a_f_2f_callacosh_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acosh_a_f_4f_callacosh_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = acosh(a);
  } SH_END;

  acosh_a_f_4f_callacosh_test2.name("acosh_a_f_4f_callacosh_test2");
  last_test = "acosh_a_f_4f_callacosh_test2";
  {  // f_4f_callacosh_test2
    ShAttrib<4, SH_CONST, float> a(float(1.3), float(2.5), float(3.8), float(10.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.756432910857), float(1.56679923697), float(2.01036749136), float(3.0797990971));

    try {
      if (test.run(acosh_a_f_4f_callacosh_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram acosh_a_f_3f_callacosh_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = acosh(a);
  } SH_END;

  acosh_a_f_3f_callacosh_test3.name("acosh_a_f_3f_callacosh_test3");
  last_test = "acosh_a_f_3f_callacosh_test3";
  {  // f_3f_callacosh_test3
    ShAttrib<3, SH_CONST, float> a(float(1.001), float(1.01), float(1.1));
    ShAttrib<3, SH_CONST, float> expected(float(0.0447176336083), float(0.141303769486), float(0.443568254385));

    try {
      if (test.run(acosh_a_f_3f_callacosh_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sinh_a_f_4f_callsinh_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = sinh(a);
  } SH_END;

  sinh_a_f_4f_callsinh_test0.name("sinh_a_f_4f_callsinh_test0");
  last_test = "sinh_a_f_4f_callsinh_test0";
  {  // f_4f_callsinh_test0
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(1.17520119364), float(3.62686040785), float(10.0178749274));

    try {
      if (test.run(sinh_a_f_4f_callsinh_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sinh_a_f_4f_callsinh_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = sinh(a);
  } SH_END;

  sinh_a_f_4f_callsinh_test1.name("sinh_a_f_4f_callsinh_test1");
  last_test = "sinh_a_f_4f_callsinh_test1";
  {  // f_4f_callsinh_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.304520293447), float(0.521095305494), float(0.888105982188), float(1.02651672571));

    try {
      if (test.run(sinh_a_f_4f_callsinh_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sinh_a_f_3f_callsinh_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = sinh(a);
  } SH_END;

  sinh_a_f_3f_callsinh_test2.name("sinh_a_f_3f_callsinh_test2");
  last_test = "sinh_a_f_3f_callsinh_test2";
  {  // f_3f_callsinh_test2
    ShAttrib<3, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(11.5487393573), float(-11.5487393573), float(0.0));

    try {
      if (test.run(sinh_a_f_3f_callsinh_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sinh_a_f_2f_callsinh_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = sinh(a);
  } SH_END;

  sinh_a_f_2f_callsinh_test3.name("sinh_a_f_2f_callsinh_test3");
  last_test = "sinh_a_f_2f_callsinh_test3";
  {  // f_2f_callsinh_test3
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(-6.28318530718));
    ShAttrib<2, SH_CONST, float> expected(float(267.744894041), float(-267.744894041));

    try {
      if (test.run(sinh_a_f_2f_callsinh_test3, a, expected, 0.06) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sinh_a_f_2f_callsinh_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = sinh(a);
  } SH_END;

  sinh_a_f_2f_callsinh_test4.name("sinh_a_f_2f_callsinh_test4");
  last_test = "sinh_a_f_2f_callsinh_test4";
  {  // f_2f_callsinh_test4
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(2.30129890231), float(-2.30129890231));

    try {
      if (test.run(sinh_a_f_2f_callsinh_test4, a, expected, 0.0005) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sinh_a_f_1f_callsinh_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = sinh(a);
  } SH_END;

  sinh_a_f_1f_callsinh_test5.name("sinh_a_f_1f_callsinh_test5");
  last_test = "sinh_a_f_1f_callsinh_test5";
  {  // f_1f_callsinh_test5
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(10.0178749274));

    try {
      if (test.run(sinh_a_f_1f_callsinh_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sinh_a_f_4f_callsinh_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = sinh(a);
  } SH_END;

  sinh_a_f_4f_callsinh_test6.name("sinh_a_f_4f_callsinh_test6");
  last_test = "sinh_a_f_4f_callsinh_test6";
  {  // f_4f_callsinh_test6
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(-0.521095305494), float(-1.17520119364), float(-10.0178749274), float(-27.2899171971));

    try {
      if (test.run(sinh_a_f_4f_callsinh_test6, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram sinh_a_f_3f_callsinh_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = sinh(a);
  } SH_END;

  sinh_a_f_3f_callsinh_test7.name("sinh_a_f_3f_callsinh_test7");
  last_test = "sinh_a_f_3f_callsinh_test7";
  {  // f_3f_callsinh_test7
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.521095305494), float(2.12927945509), float(6.05020448104));

    try {
      if (test.run(sinh_a_f_3f_callsinh_test7, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_3f_callasinh_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_3f_callasinh_test0.name("asinh_a_f_3f_callasinh_test0");
  last_test = "asinh_a_f_3f_callasinh_test0";
  {  // f_3f_callasinh_test0
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(2.0), float(3.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.88137358702), float(1.44363547518), float(1.81844645923));

    try {
      if (test.run(asinh_a_f_3f_callasinh_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_4f_callasinh_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_4f_callasinh_test1.name("asinh_a_f_4f_callasinh_test1");
  last_test = "asinh_a_f_4f_callasinh_test1";
  {  // f_4f_callasinh_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.295673047563), float(0.48121182506), float(0.732668256045), float(0.808866935653));

    try {
      if (test.run(asinh_a_f_4f_callasinh_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_2f_callasinh_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_2f_callasinh_test2.name("asinh_a_f_2f_callasinh_test2");
  last_test = "asinh_a_f_2f_callasinh_test2";
  {  // f_2f_callasinh_test2
    ShAttrib<2, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359));
    ShAttrib<2, SH_CONST, float> expected(float(1.86229574331), float(-1.86229574331));

    try {
      if (test.run(asinh_a_f_2f_callasinh_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_2f_callasinh_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_2f_callasinh_test3.name("asinh_a_f_2f_callasinh_test3");
  last_test = "asinh_a_f_2f_callasinh_test3";
  {  // f_2f_callasinh_test3
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(2.53729750137), float(1.23340311751));

    try {
      if (test.run(asinh_a_f_2f_callasinh_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_2f_callasinh_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_2f_callasinh_test4.name("asinh_a_f_2f_callasinh_test4");
  last_test = "asinh_a_f_2f_callasinh_test4";
  {  // f_2f_callasinh_test4
    ShAttrib<2, SH_CONST, float> a(float(-6.28318530718), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(-2.53729750137), float(-1.23340311751));

    try {
      if (test.run(asinh_a_f_2f_callasinh_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_1f_callasinh_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_1f_callasinh_test5.name("asinh_a_f_1f_callasinh_test5");
  last_test = "asinh_a_f_1f_callasinh_test5";
  {  // f_1f_callasinh_test5
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(1.81844645923));

    try {
      if (test.run(asinh_a_f_1f_callasinh_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_1f_callasinh_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_1f_callasinh_test6.name("asinh_a_f_1f_callasinh_test6");
  last_test = "asinh_a_f_1f_callasinh_test6";
  {  // f_1f_callasinh_test6
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(4.4408920985e-16));

    try {
      if (test.run(asinh_a_f_1f_callasinh_test6, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_4f_callasinh_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_4f_callasinh_test7.name("asinh_a_f_4f_callasinh_test7");
  last_test = "asinh_a_f_4f_callasinh_test7";
  {  // f_4f_callasinh_test7
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(-0.48121182506), float(-0.88137358702), float(-1.81844645923), float(-2.09471254726));

    try {
      if (test.run(asinh_a_f_4f_callasinh_test7, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram asinh_a_f_3f_callasinh_test8 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = asinh(a);
  } SH_END;

  asinh_a_f_3f_callasinh_test8.name("asinh_a_f_3f_callasinh_test8");
  last_test = "asinh_a_f_3f_callasinh_test8";
  {  // f_3f_callasinh_test8
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.48121182506), float(1.19476321729), float(1.64723114637));

    try {
      if (test.run(asinh_a_f_3f_callasinh_test8, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tanh_a_f_4f_calltanh_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = tanh(a);
  } SH_END;

  tanh_a_f_4f_calltanh_test0.name("tanh_a_f_4f_calltanh_test0");
  last_test = "tanh_a_f_4f_calltanh_test0";
  {  // f_4f_calltanh_test0
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(0.761594155956), float(0.964027580076), float(0.995054753687));

    try {
      if (test.run(tanh_a_f_4f_calltanh_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tanh_a_f_4f_calltanh_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = tanh(a);
  } SH_END;

  tanh_a_f_4f_calltanh_test1.name("tanh_a_f_4f_calltanh_test1");
  last_test = "tanh_a_f_4f_calltanh_test1";
  {  // f_4f_calltanh_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.291312612452), float(0.46211715726), float(0.664036770268), float(0.716297870199));

    try {
      if (test.run(tanh_a_f_4f_calltanh_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tanh_a_f_3f_calltanh_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = tanh(a);
  } SH_END;

  tanh_a_f_3f_calltanh_test2.name("tanh_a_f_3f_calltanh_test2");
  last_test = "tanh_a_f_3f_calltanh_test2";
  {  // f_3f_calltanh_test2
    ShAttrib<3, SH_CONST, float> a(float(1.47079632679), float(-1.47079632679), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.899729252467), float(-0.899729252467), float(0.0));

    try {
      if (test.run(tanh_a_f_3f_calltanh_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tanh_a_f_3f_calltanh_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = tanh(a);
  } SH_END;

  tanh_a_f_3f_calltanh_test3.name("tanh_a_f_3f_calltanh_test3");
  last_test = "tanh_a_f_3f_calltanh_test3";
  {  // f_3f_calltanh_test3
    ShAttrib<3, SH_CONST, float> a(float(-0.5), float(-1.0), float(-1.1));
    ShAttrib<3, SH_CONST, float> expected(float(-0.46211715726), float(-0.761594155956), float(-0.800499021761));

    try {
      if (test.run(tanh_a_f_3f_calltanh_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram tanh_a_f_3f_calltanh_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = tanh(a);
  } SH_END;

  tanh_a_f_3f_calltanh_test4.name("tanh_a_f_3f_calltanh_test4");
  last_test = "tanh_a_f_3f_calltanh_test4";
  {  // f_3f_calltanh_test4
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.46211715726), float(0.905148253645), float(0.986614298151));

    try {
      if (test.run(tanh_a_f_3f_calltanh_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atanh_a_f_1f_callatanh_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = atanh(a);
  } SH_END;

  atanh_a_f_1f_callatanh_test0.name("atanh_a_f_1f_callatanh_test0");
  last_test = "atanh_a_f_1f_callatanh_test0";
  {  // f_1f_callatanh_test0
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    try {
      if (test.run(atanh_a_f_1f_callatanh_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atanh_a_f_4f_callatanh_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = atanh(a);
  } SH_END;

  atanh_a_f_4f_callatanh_test1.name("atanh_a_f_4f_callatanh_test1");
  last_test = "atanh_a_f_4f_callatanh_test1";
  {  // f_4f_callatanh_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.309519604203), float(0.549306144334), float(1.09861228867), float(1.47221948958));

    try {
      if (test.run(atanh_a_f_4f_callatanh_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atanh_a_f_2f_callatanh_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atanh(a);
  } SH_END;

  atanh_a_f_2f_callatanh_test2.name("atanh_a_f_2f_callatanh_test2");
  last_test = "atanh_a_f_2f_callatanh_test2";
  {  // f_2f_callatanh_test2
    ShAttrib<2, SH_CONST, float> a(float(0.9), float(0.0));
    ShAttrib<2, SH_CONST, float> expected(float(1.47221948958), float(0.0));

    try {
      if (test.run(atanh_a_f_2f_callatanh_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atanh_a_f_1f_callatanh_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = atanh(a);
  } SH_END;

  atanh_a_f_1f_callatanh_test3.name("atanh_a_f_1f_callatanh_test3");
  last_test = "atanh_a_f_1f_callatanh_test3";
  {  // f_1f_callatanh_test3
    ShAttrib<1, SH_CONST, float> a(float(-0.9));
    ShAttrib<1, SH_CONST, float> expected(float(-1.47221948958));

    try {
      if (test.run(atanh_a_f_1f_callatanh_test3, a, expected, 0.09) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atanh_a_f_2f_callatanh_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atanh(a);
  } SH_END;

  atanh_a_f_2f_callatanh_test4.name("atanh_a_f_2f_callatanh_test4");
  last_test = "atanh_a_f_2f_callatanh_test4";
  {  // f_2f_callatanh_test4
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-0.1));
    ShAttrib<2, SH_CONST, float> expected(float(-0.549306144334), float(-0.100335347731));

    try {
      if (test.run(atanh_a_f_2f_callatanh_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atanh_a_f_3f_callatanh_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = atanh(a);
  } SH_END;

  atanh_a_f_3f_callatanh_test5.name("atanh_a_f_3f_callatanh_test5");
  last_test = "atanh_a_f_3f_callatanh_test5";
  {  // f_3f_callatanh_test5
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(0.6), float(0.9));
    ShAttrib<3, SH_CONST, float> expected(float(0.549306144334), float(0.69314718056), float(1.47221948958));

    try {
      if (test.run(atanh_a_f_3f_callatanh_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_3f_callatan_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_3f_callatan_test0.name("atan_a_f_3f_callatan_test0");
  last_test = "atan_a_f_3f_callatan_test0";
  {  // f_3f_callatan_test0
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(2.0), float(3.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.785398163397), float(1.10714871779), float(1.2490457724));

    try {
      if (test.run(atan_a_f_3f_callatan_test0, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_4f_callatan_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_4f_callatan_test1.name("atan_a_f_4f_callatan_test1");
  last_test = "atan_a_f_4f_callatan_test1";
  {  // f_4f_callatan_test1
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.291456794478), float(0.463647609001), float(0.674740942224), float(0.732815101787));

    try {
      if (test.run(atan_a_f_4f_callatan_test1, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_2f_callatan_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_2f_callatan_test2.name("atan_a_f_2f_callatan_test2");
  last_test = "atan_a_f_2f_callatan_test2";
  {  // f_2f_callatan_test2
    ShAttrib<2, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359));
    ShAttrib<2, SH_CONST, float> expected(float(1.26262725568), float(-1.26262725568));

    try {
      if (test.run(atan_a_f_2f_callatan_test2, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_2f_callatan_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_2f_callatan_test3.name("atan_a_f_2f_callatan_test3");
  last_test = "atan_a_f_2f_callatan_test3";
  {  // f_2f_callatan_test3
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(1.41296513651), float(1.00388482185));

    try {
      if (test.run(atan_a_f_2f_callatan_test3, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_2f_callatan_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_2f_callatan_test4.name("atan_a_f_2f_callatan_test4");
  last_test = "atan_a_f_2f_callatan_test4";
  {  // f_2f_callatan_test4
    ShAttrib<2, SH_CONST, float> a(float(-6.28318530718), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(-1.41296513651), float(-1.00388482185));

    try {
      if (test.run(atan_a_f_2f_callatan_test4, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_1f_callatan_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_1f_callatan_test5.name("atan_a_f_1f_callatan_test5");
  last_test = "atan_a_f_1f_callatan_test5";
  {  // f_1f_callatan_test5
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(1.2490457724));

    try {
      if (test.run(atan_a_f_1f_callatan_test5, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_1f_callatan_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_1f_callatan_test6.name("atan_a_f_1f_callatan_test6");
  last_test = "atan_a_f_1f_callatan_test6";
  {  // f_1f_callatan_test6
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    try {
      if (test.run(atan_a_f_1f_callatan_test6, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_4f_callatan_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_4f_callatan_test7.name("atan_a_f_4f_callatan_test7");
  last_test = "atan_a_f_4f_callatan_test7";
  {  // f_4f_callatan_test7
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(-0.463647609001), float(-0.785398163397), float(-1.2490457724), float(-1.32581766367));

    try {
      if (test.run(atan_a_f_4f_callatan_test7, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan_a_f_3f_callatan_test8 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = atan(a);
  } SH_END;

  atan_a_f_3f_callatan_test8.name("atan_a_f_3f_callatan_test8");
  last_test = "atan_a_f_3f_callatan_test8";
  {  // f_3f_callatan_test8
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.463647609001), float(0.982793723247), float(1.19028994968));

    try {
      if (test.run(atan_a_f_3f_callatan_test8, a, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_4f_4f_callatan2_test0 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_4f_4f_callatan2_test0.name("atan2_a_f_4f_4f_callatan2_test0");
  last_test = "atan2_a_f_4f_4f_callatan2_test0";
  {  // f_4f_4f_callatan2_test0
    ShAttrib<4, SH_CONST, float> a(float(1.0), float(6.28318530718), float(50), float(1.0));
    ShAttrib<4, SH_CONST, float> b(float(0.3), float(1.0), float(3.0), float(-6.28318530718));
    ShAttrib<4, SH_CONST, float> expected(float(1.27933953232), float(1.41296513651), float(1.51086817167), float(2.9837614633));

    try {
      if (test.run(atan2_a_f_4f_4f_callatan2_test0, a, b, expected, 0.015) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_2f_2f_callatan2_test1 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_INPUT, float> b;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_2f_2f_callatan2_test1.name("atan2_a_f_2f_2f_callatan2_test1");
  last_test = "atan2_a_f_2f_2f_callatan2_test1";
  {  // f_2f_2f_callatan2_test1
    ShAttrib<2, SH_CONST, float> a(float(1.0), float(1.0));
    ShAttrib<2, SH_CONST, float> b(float(1.0), float(2.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.785398163397), float(0.463647609001));

    try {
      if (test.run(atan2_a_f_2f_2f_callatan2_test1, a, b, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_2f_2f_callatan2_test2 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_INPUT, float> b;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_2f_2f_callatan2_test2.name("atan2_a_f_2f_2f_callatan2_test2");
  last_test = "atan2_a_f_2f_2f_callatan2_test2";
  {  // f_2f_2f_callatan2_test2
    ShAttrib<2, SH_CONST, float> a(float(1.5), float(3.0));
    ShAttrib<2, SH_CONST, float> b(float(1.0), float(2.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.982793723247), float(0.982793723247));

    try {
      if (test.run(atan2_a_f_2f_2f_callatan2_test2, a, b, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_4f_4f_callatan2_test3 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_4f_4f_callatan2_test3.name("atan2_a_f_4f_4f_callatan2_test3");
  last_test = "atan2_a_f_4f_4f_callatan2_test3";
  {  // f_4f_4f_callatan2_test3
    ShAttrib<4, SH_CONST, float> a(float(0), float(0), float(0), float(0));
    ShAttrib<4, SH_CONST, float> b(float(0.5), float(1.0), float(2.0), float(-1.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(0.0), float(0.0), float(3.14159265359));

    try {
      if (test.run(atan2_a_f_4f_4f_callatan2_test3, a, b, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_3f_3f_callatan2_test4 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_3f_3f_callatan2_test4.name("atan2_a_f_3f_3f_callatan2_test4");
  last_test = "atan2_a_f_3f_3f_callatan2_test4";
  {  // f_3f_3f_callatan2_test4
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> b(float(0.5), float(0.8), float(0.9));
    ShAttrib<3, SH_CONST, float> expected(float(1.10714871779), float(0.896055384571), float(0.837981225008));

    try {
      if (test.run(atan2_a_f_3f_3f_callatan2_test4, a, b, expected, 0.002) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_2f_2f_callatan2_test5 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_INPUT, float> b;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_2f_2f_callatan2_test5.name("atan2_a_f_2f_2f_callatan2_test5");
  last_test = "atan2_a_f_2f_2f_callatan2_test5";
  {  // f_2f_2f_callatan2_test5
    ShAttrib<2, SH_CONST, float> a(float(1.0), float(1.0));
    ShAttrib<2, SH_CONST, float> b(float(3.14159265359), float(-3.14159265359));
    ShAttrib<2, SH_CONST, float> expected(float(0.308169071116), float(2.83342358247));

    try {
      if (test.run(atan2_a_f_2f_2f_callatan2_test5, a, b, expected, 0.004) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_2f_2f_callatan2_test6 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_INPUT, float> b;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_2f_2f_callatan2_test6.name("atan2_a_f_2f_2f_callatan2_test6");
  last_test = "atan2_a_f_2f_2f_callatan2_test6";
  {  // f_2f_2f_callatan2_test6
    ShAttrib<2, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359));
    ShAttrib<2, SH_CONST, float> b(float(1.0), float(1.0));
    ShAttrib<2, SH_CONST, float> expected(float(1.26262725568), float(-1.26262725568));

    try {
      if (test.run(atan2_a_f_2f_2f_callatan2_test6, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_2f_2f_callatan2_test7 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_INPUT, float> b;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_2f_2f_callatan2_test7.name("atan2_a_f_2f_2f_callatan2_test7");
  last_test = "atan2_a_f_2f_2f_callatan2_test7";
  {  // f_2f_2f_callatan2_test7
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(1.57079632679));
    ShAttrib<2, SH_CONST, float> b(float(-6.28318530718), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(2.35619449019), float(2.35619449019));

    try {
      if (test.run(atan2_a_f_2f_2f_callatan2_test7, a, b, expected, 0.002) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_2f_2f_callatan2_test8 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<2, SH_INPUT, float> a;
    ShAttrib<2, SH_INPUT, float> b;
    ShAttrib<2, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_2f_2f_callatan2_test8.name("atan2_a_f_2f_2f_callatan2_test8");
  last_test = "atan2_a_f_2f_2f_callatan2_test8";
  {  // f_2f_2f_callatan2_test8
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(1.0));
    ShAttrib<2, SH_CONST, float> b(float(1.0), float(1.0));
    ShAttrib<2, SH_CONST, float> expected(float(1.00388482185), float(0.785398163397));

    try {
      if (test.run(atan2_a_f_2f_2f_callatan2_test8, a, b, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_1f_1f_callatan2_test9 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_INPUT, float> b;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_1f_1f_callatan2_test9.name("atan2_a_f_1f_1f_callatan2_test9");
  last_test = "atan2_a_f_1f_1f_callatan2_test9";
  {  // f_1f_1f_callatan2_test9
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> b(float(-1.57079632679));
    ShAttrib<1, SH_CONST, float> expected(float(2.57468114865));

    try {
      if (test.run(atan2_a_f_1f_1f_callatan2_test9, a, b, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_1f_1f_callatan2_test10 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<1, SH_INPUT, float> a;
    ShAttrib<1, SH_INPUT, float> b;
    ShAttrib<1, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_1f_1f_callatan2_test10.name("atan2_a_f_1f_1f_callatan2_test10");
  last_test = "atan2_a_f_1f_1f_callatan2_test10";
  {  // f_1f_1f_callatan2_test10
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> b(float(9.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.321750554397));

    try {
      if (test.run(atan2_a_f_1f_1f_callatan2_test10, a, b, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_4f_4f_callatan2_test11 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_4f_4f_callatan2_test11.name("atan2_a_f_4f_4f_callatan2_test11");
  last_test = "atan2_a_f_4f_4f_callatan2_test11";
  {  // f_4f_4f_callatan2_test11
    ShAttrib<4, SH_CONST, float> a(float(0.5), float(-1.0), float(-3.0), float(-3.0));
    ShAttrib<4, SH_CONST, float> b(float(-1.0), float(1.0), float(1.0), float(-1.0));
    ShAttrib<4, SH_CONST, float> expected(float(2.67794504459), float(-0.785398163397), float(-1.2490457724), float(-1.89254688119));

    try {
      if (test.run(atan2_a_f_4f_4f_callatan2_test11, a, b, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_3f_3f_callatan2_test12 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_3f_3f_callatan2_test12.name("atan2_a_f_3f_3f_callatan2_test12");
  last_test = "atan2_a_f_3f_3f_callatan2_test12";
  {  // f_3f_3f_callatan2_test12
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> b(float(1.0), float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.463647609001), float(0.982793723247), float(1.19028994968));

    try {
      if (test.run(atan2_a_f_3f_3f_callatan2_test12, a, b, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_3f_3f_callatan2_test13 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<3, SH_INPUT, float> a;
    ShAttrib<3, SH_INPUT, float> b;
    ShAttrib<3, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_3f_3f_callatan2_test13.name("atan2_a_f_3f_3f_callatan2_test13");
  last_test = "atan2_a_f_3f_3f_callatan2_test13";
  {  // f_3f_3f_callatan2_test13
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> b(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(1.10714871779), float(0.588002603548), float(0.380506377112));

    try {
      if (test.run(atan2_a_f_3f_3f_callatan2_test13, a, b, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  ShProgram atan2_a_f_4f_4f_callatan2_test14 = SH_BEGIN_PROGRAM("stream") {
    ShAttrib<4, SH_INPUT, float> a;
    ShAttrib<4, SH_INPUT, float> b;
    ShAttrib<4, SH_OUTPUT, float> out;
    out = atan2(a, b);
  } SH_END;

  atan2_a_f_4f_4f_callatan2_test14.name("atan2_a_f_4f_4f_callatan2_test14");
  last_test = "atan2_a_f_4f_4f_callatan2_test14";
  {  // f_4f_4f_callatan2_test14
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.34), float(1.5), float(-2.5));
    ShAttrib<4, SH_CONST, float> b(float(0.0), float(0.0), float(0.0), float(0.0));
    ShAttrib<4, SH_CONST, float> expected(float(1.57079632679), float(1.57079632679), float(1.57079632679), float(-1.57079632679));

    try {
      if (test.run(atan2_a_f_4f_4f_callatan2_test14, a, b, expected, 0.0001) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }
    total_tests++;
  }

  {  // f_4f_callcos_test0
    last_test = "f_4f_callcos_test0";
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(1.0), float(0.540302305868), float(-0.416146836547), float(-0.9899924966));

    ShAttrib<4, SH_TEMP, float> out;
    out = cos(a);

    try {
      if (test.check("f_4f_callcos_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callcos_test1
    last_test = "f_4f_callcos_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.955336489126), float(0.87758256189), float(0.696706709347), float(0.621609968271));

    ShAttrib<4, SH_TEMP, float> out;
    out = cos(a);

    try {
      if (test.check("f_4f_callcos_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callcos_test2
    last_test = "f_3f_callcos_test2";
    ShAttrib<3, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(-1.0), float(-1.0), float(1.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = cos(a);

    try {
      if (test.check("f_3f_callcos_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callcos_test3
    last_test = "f_2f_callcos_test3";
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(-6.28318530718));
    ShAttrib<2, SH_CONST, float> expected(float(1.0), float(1.0));

    ShAttrib<2, SH_TEMP, float> out;
    out = cos(a);

    try {
      if (test.check("f_2f_callcos_test3", out, expected, 0.06) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callcos_test4
    last_test = "f_2f_callcos_test4";
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(6.12303176911e-17), float(6.12303176911e-17));

    ShAttrib<2, SH_TEMP, float> out;
    out = cos(a);

    try {
      if (test.check("f_2f_callcos_test4", out, expected, 0.0005) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callcos_test5
    last_test = "f_1f_callcos_test5";
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(-0.9899924966));

    ShAttrib<1, SH_TEMP, float> out;
    out = cos(a);

    try {
      if (test.check("f_1f_callcos_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callcos_test6
    last_test = "f_4f_callcos_test6";
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.87758256189), float(0.540302305868), float(-0.9899924966), float(-0.653643620864));

    ShAttrib<4, SH_TEMP, float> out;
    out = cos(a);

    try {
      if (test.check("f_4f_callcos_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callcos_test7
    last_test = "f_3f_callcos_test7";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.87758256189), float(0.0707372016677), float(-0.801143615547));

    ShAttrib<3, SH_TEMP, float> out;
    out = cos(a);

    try {
      if (test.check("f_3f_callcos_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callacos_test0
    last_test = "f_1f_callacos_test0";
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(1.57079632679));

    ShAttrib<1, SH_TEMP, float> out;
    out = acos(a);

    try {
      if (test.check("f_1f_callacos_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callacos_test1
    last_test = "f_4f_callacos_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(1.26610367278), float(1.0471975512), float(0.643501108793), float(0.451026811796));

    ShAttrib<4, SH_TEMP, float> out;
    out = acos(a);

    try {
      if (test.check("f_4f_callacos_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callacos_test2
    last_test = "f_2f_callacos_test2";
    ShAttrib<2, SH_CONST, float> a(float(0.9), float(0.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.451026811796), float(1.57079632679));

    ShAttrib<2, SH_TEMP, float> out;
    out = acos(a);

    try {
      if (test.check("f_2f_callacos_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callacos_test3
    last_test = "f_1f_callacos_test3";
    ShAttrib<1, SH_CONST, float> a(float(-0.9));
    ShAttrib<1, SH_CONST, float> expected(float(2.69056584179));

    ShAttrib<1, SH_TEMP, float> out;
    out = acos(a);

    try {
      if (test.check("f_1f_callacos_test3", out, expected, 0.09) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callacos_test4
    last_test = "f_2f_callacos_test4";
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-0.1));
    ShAttrib<2, SH_CONST, float> expected(float(2.09439510239), float(1.67096374796));

    ShAttrib<2, SH_TEMP, float> out;
    out = acos(a);

    try {
      if (test.check("f_2f_callacos_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callacos_test5
    last_test = "f_3f_callacos_test5";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(0.6), float(0.9));
    ShAttrib<3, SH_CONST, float> expected(float(1.0471975512), float(0.927295218002), float(0.451026811796));

    ShAttrib<3, SH_TEMP, float> out;
    out = acos(a);

    try {
      if (test.check("f_3f_callacos_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callsin_test0
    last_test = "f_4f_callsin_test0";
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(0.841470984808), float(0.909297426826), float(0.14112000806));

    ShAttrib<4, SH_TEMP, float> out;
    out = sin(a);

    try {
      if (test.check("f_4f_callsin_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callsin_test1
    last_test = "f_4f_callsin_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.295520206661), float(0.479425538604), float(0.7173560909), float(0.783326909627));

    ShAttrib<4, SH_TEMP, float> out;
    out = sin(a);

    try {
      if (test.check("f_4f_callsin_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callsin_test2
    last_test = "f_3f_callsin_test2";
    ShAttrib<3, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(1.22460635382e-16), float(-1.22460635382e-16), float(0.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = sin(a);

    try {
      if (test.check("f_3f_callsin_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callsin_test3
    last_test = "f_2f_callsin_test3";
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(-6.28318530718));
    ShAttrib<2, SH_CONST, float> expected(float(-2.44921270764e-16), float(2.44921270764e-16));

    ShAttrib<2, SH_TEMP, float> out;
    out = sin(a);

    try {
      if (test.check("f_2f_callsin_test3", out, expected, 0.06) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callsin_test4
    last_test = "f_2f_callsin_test4";
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(1.0), float(-1.0));

    ShAttrib<2, SH_TEMP, float> out;
    out = sin(a);

    try {
      if (test.check("f_2f_callsin_test4", out, expected, 0.0005) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callsin_test5
    last_test = "f_1f_callsin_test5";
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.14112000806));

    ShAttrib<1, SH_TEMP, float> out;
    out = sin(a);

    try {
      if (test.check("f_1f_callsin_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callsin_test6
    last_test = "f_4f_callsin_test6";
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(-0.479425538604), float(-0.841470984808), float(-0.14112000806), float(0.756802495308));

    ShAttrib<4, SH_TEMP, float> out;
    out = sin(a);

    try {
      if (test.check("f_4f_callsin_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callsin_test7
    last_test = "f_3f_callsin_test7";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.479425538604), float(0.997494986604), float(0.598472144104));

    ShAttrib<3, SH_TEMP, float> out;
    out = sin(a);

    try {
      if (test.check("f_3f_callsin_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callasin_test0
    last_test = "f_1f_callasin_test0";
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = asin(a);

    try {
      if (test.check("f_1f_callasin_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callasin_test1
    last_test = "f_4f_callasin_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.304692654015), float(0.523598775598), float(0.927295218002), float(1.119769515));

    ShAttrib<4, SH_TEMP, float> out;
    out = asin(a);

    try {
      if (test.check("f_4f_callasin_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callasin_test2
    last_test = "f_2f_callasin_test2";
    ShAttrib<2, SH_CONST, float> a(float(0.9), float(0.0));
    ShAttrib<2, SH_CONST, float> expected(float(1.119769515), float(0.0));

    ShAttrib<2, SH_TEMP, float> out;
    out = asin(a);

    try {
      if (test.check("f_2f_callasin_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callasin_test3
    last_test = "f_1f_callasin_test3";
    ShAttrib<1, SH_CONST, float> a(float(-0.9));
    ShAttrib<1, SH_CONST, float> expected(float(-1.119769515));

    ShAttrib<1, SH_TEMP, float> out;
    out = asin(a);

    try {
      if (test.check("f_1f_callasin_test3", out, expected, 0.09) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callasin_test4
    last_test = "f_2f_callasin_test4";
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-0.1));
    ShAttrib<2, SH_CONST, float> expected(float(-0.523598775598), float(-0.100167421162));

    ShAttrib<2, SH_TEMP, float> out;
    out = asin(a);

    try {
      if (test.check("f_2f_callasin_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callasin_test5
    last_test = "f_3f_callasin_test5";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(0.6), float(0.9));
    ShAttrib<3, SH_CONST, float> expected(float(0.523598775598), float(0.643501108793), float(1.119769515));

    ShAttrib<3, SH_TEMP, float> out;
    out = asin(a);

    try {
      if (test.check("f_3f_callasin_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_calltan_test0
    last_test = "f_4f_calltan_test0";
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(1.55740772465), float(-2.18503986326), float(-0.142546543074));

    ShAttrib<4, SH_TEMP, float> out;
    out = tan(a);

    try {
      if (test.check("f_4f_calltan_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_calltan_test1
    last_test = "f_4f_calltan_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.30933624961), float(0.546302489844), float(1.02963855705), float(1.26015821755));

    ShAttrib<4, SH_TEMP, float> out;
    out = tan(a);

    try {
      if (test.check("f_4f_calltan_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltan_test2
    last_test = "f_3f_calltan_test2";
    ShAttrib<3, SH_CONST, float> a(float(1.47079632679), float(-1.47079632679), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(9.96664442326), float(-9.96664442326), float(0.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = tan(a);

    try {
      if (test.check("f_3f_calltan_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltan_test3
    last_test = "f_3f_calltan_test3";
    ShAttrib<3, SH_CONST, float> a(float(-0.5), float(-1.0), float(-1.1));
    ShAttrib<3, SH_CONST, float> expected(float(-0.546302489844), float(-1.55740772465), float(-1.96475965725));

    ShAttrib<3, SH_TEMP, float> out;
    out = tan(a);

    try {
      if (test.check("f_3f_calltan_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltan_test4
    last_test = "f_3f_calltan_test4";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.546302489844), float(14.1014199472), float(-0.747022297239));

    ShAttrib<3, SH_TEMP, float> out;
    out = tan(a);

    try {
      if (test.check("f_3f_calltan_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callcosh_test0
    last_test = "f_4f_callcosh_test0";
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(1.0), float(1.54308063482), float(3.76219569108), float(10.0676619958));

    ShAttrib<4, SH_TEMP, float> out;
    out = cosh(a);

    try {
      if (test.check("f_4f_callcosh_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callcosh_test1
    last_test = "f_4f_callcosh_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(1.04533851413), float(1.12762596521), float(1.3374349463), float(1.43308638545));

    ShAttrib<4, SH_TEMP, float> out;
    out = cosh(a);

    try {
      if (test.check("f_4f_callcosh_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callcosh_test2
    last_test = "f_3f_callcosh_test2";
    ShAttrib<3, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(11.5919532755), float(11.5919532755), float(1.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = cosh(a);

    try {
      if (test.check("f_3f_callcosh_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callcosh_test3
    last_test = "f_2f_callcosh_test3";
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(-6.28318530718));
    ShAttrib<2, SH_CONST, float> expected(float(267.746761484), float(267.746761484));

    ShAttrib<2, SH_TEMP, float> out;
    out = cosh(a);

    try {
      if (test.check("f_2f_callcosh_test3", out, expected, 0.06) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callcosh_test4
    last_test = "f_2f_callcosh_test4";
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(2.50917847866), float(2.50917847866));

    ShAttrib<2, SH_TEMP, float> out;
    out = cosh(a);

    try {
      if (test.check("f_2f_callcosh_test4", out, expected, 0.0005) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callcosh_test5
    last_test = "f_1f_callcosh_test5";
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(10.0676619958));

    ShAttrib<1, SH_TEMP, float> out;
    out = cosh(a);

    try {
      if (test.check("f_1f_callcosh_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callcosh_test6
    last_test = "f_4f_callcosh_test6";
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(1.12762596521), float(1.54308063482), float(10.0676619958), float(27.308232836));

    ShAttrib<4, SH_TEMP, float> out;
    out = cosh(a);

    try {
      if (test.check("f_4f_callcosh_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callcosh_test7
    last_test = "f_3f_callcosh_test7";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(1.12762596521), float(2.35240961524), float(6.13228947966));

    ShAttrib<3, SH_TEMP, float> out;
    out = cosh(a);

    try {
      if (test.check("f_3f_callcosh_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callacosh_test0
    last_test = "f_1f_callacosh_test0";
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> expected(float(4.4408920985e-16));

    ShAttrib<1, SH_TEMP, float> out;
    out = acosh(a);

    try {
      if (test.check("f_1f_callacosh_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callacosh_test1
    last_test = "f_2f_callacosh_test1";
    ShAttrib<2, SH_CONST, float> a(float(50.3), float(2000.5));
    ShAttrib<2, SH_CONST, float> expected(float(4.6110534323), float(8.29429954639));

    ShAttrib<2, SH_TEMP, float> out;
    out = acosh(a);

    try {
      if (test.check("f_2f_callacosh_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callacosh_test2
    last_test = "f_4f_callacosh_test2";
    ShAttrib<4, SH_CONST, float> a(float(1.3), float(2.5), float(3.8), float(10.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.756432910857), float(1.56679923697), float(2.01036749136), float(3.0797990971));

    ShAttrib<4, SH_TEMP, float> out;
    out = acosh(a);

    try {
      if (test.check("f_4f_callacosh_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callacosh_test3
    last_test = "f_3f_callacosh_test3";
    ShAttrib<3, SH_CONST, float> a(float(1.001), float(1.01), float(1.1));
    ShAttrib<3, SH_CONST, float> expected(float(0.0447176336083), float(0.141303769486), float(0.443568254385));

    ShAttrib<3, SH_TEMP, float> out;
    out = acosh(a);

    try {
      if (test.check("f_3f_callacosh_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callsinh_test0
    last_test = "f_4f_callsinh_test0";
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(1.17520119364), float(3.62686040785), float(10.0178749274));

    ShAttrib<4, SH_TEMP, float> out;
    out = sinh(a);

    try {
      if (test.check("f_4f_callsinh_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callsinh_test1
    last_test = "f_4f_callsinh_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.304520293447), float(0.521095305494), float(0.888105982188), float(1.02651672571));

    ShAttrib<4, SH_TEMP, float> out;
    out = sinh(a);

    try {
      if (test.check("f_4f_callsinh_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callsinh_test2
    last_test = "f_3f_callsinh_test2";
    ShAttrib<3, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(11.5487393573), float(-11.5487393573), float(0.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = sinh(a);

    try {
      if (test.check("f_3f_callsinh_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callsinh_test3
    last_test = "f_2f_callsinh_test3";
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(-6.28318530718));
    ShAttrib<2, SH_CONST, float> expected(float(267.744894041), float(-267.744894041));

    ShAttrib<2, SH_TEMP, float> out;
    out = sinh(a);

    try {
      if (test.check("f_2f_callsinh_test3", out, expected, 0.06) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callsinh_test4
    last_test = "f_2f_callsinh_test4";
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(2.30129890231), float(-2.30129890231));

    ShAttrib<2, SH_TEMP, float> out;
    out = sinh(a);

    try {
      if (test.check("f_2f_callsinh_test4", out, expected, 0.0005) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callsinh_test5
    last_test = "f_1f_callsinh_test5";
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(10.0178749274));

    ShAttrib<1, SH_TEMP, float> out;
    out = sinh(a);

    try {
      if (test.check("f_1f_callsinh_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callsinh_test6
    last_test = "f_4f_callsinh_test6";
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(-0.521095305494), float(-1.17520119364), float(-10.0178749274), float(-27.2899171971));

    ShAttrib<4, SH_TEMP, float> out;
    out = sinh(a);

    try {
      if (test.check("f_4f_callsinh_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callsinh_test7
    last_test = "f_3f_callsinh_test7";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.521095305494), float(2.12927945509), float(6.05020448104));

    ShAttrib<3, SH_TEMP, float> out;
    out = sinh(a);

    try {
      if (test.check("f_3f_callsinh_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callasinh_test0
    last_test = "f_3f_callasinh_test0";
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(2.0), float(3.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.88137358702), float(1.44363547518), float(1.81844645923));

    ShAttrib<3, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_3f_callasinh_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callasinh_test1
    last_test = "f_4f_callasinh_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.295673047563), float(0.48121182506), float(0.732668256045), float(0.808866935653));

    ShAttrib<4, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_4f_callasinh_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callasinh_test2
    last_test = "f_2f_callasinh_test2";
    ShAttrib<2, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359));
    ShAttrib<2, SH_CONST, float> expected(float(1.86229574331), float(-1.86229574331));

    ShAttrib<2, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_2f_callasinh_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callasinh_test3
    last_test = "f_2f_callasinh_test3";
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(2.53729750137), float(1.23340311751));

    ShAttrib<2, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_2f_callasinh_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callasinh_test4
    last_test = "f_2f_callasinh_test4";
    ShAttrib<2, SH_CONST, float> a(float(-6.28318530718), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(-2.53729750137), float(-1.23340311751));

    ShAttrib<2, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_2f_callasinh_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callasinh_test5
    last_test = "f_1f_callasinh_test5";
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(1.81844645923));

    ShAttrib<1, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_1f_callasinh_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callasinh_test6
    last_test = "f_1f_callasinh_test6";
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(4.4408920985e-16));

    ShAttrib<1, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_1f_callasinh_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callasinh_test7
    last_test = "f_4f_callasinh_test7";
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(-0.48121182506), float(-0.88137358702), float(-1.81844645923), float(-2.09471254726));

    ShAttrib<4, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_4f_callasinh_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callasinh_test8
    last_test = "f_3f_callasinh_test8";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.48121182506), float(1.19476321729), float(1.64723114637));

    ShAttrib<3, SH_TEMP, float> out;
    out = asinh(a);

    try {
      if (test.check("f_3f_callasinh_test8", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_calltanh_test0
    last_test = "f_4f_calltanh_test0";
    ShAttrib<4, SH_CONST, float> a(float(0.0), float(1.0), float(2.0), float(3.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(0.761594155956), float(0.964027580076), float(0.995054753687));

    ShAttrib<4, SH_TEMP, float> out;
    out = tanh(a);

    try {
      if (test.check("f_4f_calltanh_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_calltanh_test1
    last_test = "f_4f_calltanh_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.291312612452), float(0.46211715726), float(0.664036770268), float(0.716297870199));

    ShAttrib<4, SH_TEMP, float> out;
    out = tanh(a);

    try {
      if (test.check("f_4f_calltanh_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltanh_test2
    last_test = "f_3f_calltanh_test2";
    ShAttrib<3, SH_CONST, float> a(float(1.47079632679), float(-1.47079632679), float(0.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.899729252467), float(-0.899729252467), float(0.0));

    ShAttrib<3, SH_TEMP, float> out;
    out = tanh(a);

    try {
      if (test.check("f_3f_calltanh_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltanh_test3
    last_test = "f_3f_calltanh_test3";
    ShAttrib<3, SH_CONST, float> a(float(-0.5), float(-1.0), float(-1.1));
    ShAttrib<3, SH_CONST, float> expected(float(-0.46211715726), float(-0.761594155956), float(-0.800499021761));

    ShAttrib<3, SH_TEMP, float> out;
    out = tanh(a);

    try {
      if (test.check("f_3f_calltanh_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_calltanh_test4
    last_test = "f_3f_calltanh_test4";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.46211715726), float(0.905148253645), float(0.986614298151));

    ShAttrib<3, SH_TEMP, float> out;
    out = tanh(a);

    try {
      if (test.check("f_3f_calltanh_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callatanh_test0
    last_test = "f_1f_callatanh_test0";
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = atanh(a);

    try {
      if (test.check("f_1f_callatanh_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callatanh_test1
    last_test = "f_4f_callatanh_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.309519604203), float(0.549306144334), float(1.09861228867), float(1.47221948958));

    ShAttrib<4, SH_TEMP, float> out;
    out = atanh(a);

    try {
      if (test.check("f_4f_callatanh_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callatanh_test2
    last_test = "f_2f_callatanh_test2";
    ShAttrib<2, SH_CONST, float> a(float(0.9), float(0.0));
    ShAttrib<2, SH_CONST, float> expected(float(1.47221948958), float(0.0));

    ShAttrib<2, SH_TEMP, float> out;
    out = atanh(a);

    try {
      if (test.check("f_2f_callatanh_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callatanh_test3
    last_test = "f_1f_callatanh_test3";
    ShAttrib<1, SH_CONST, float> a(float(-0.9));
    ShAttrib<1, SH_CONST, float> expected(float(-1.47221948958));

    ShAttrib<1, SH_TEMP, float> out;
    out = atanh(a);

    try {
      if (test.check("f_1f_callatanh_test3", out, expected, 0.09) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callatanh_test4
    last_test = "f_2f_callatanh_test4";
    ShAttrib<2, SH_CONST, float> a(float(-0.5), float(-0.1));
    ShAttrib<2, SH_CONST, float> expected(float(-0.549306144334), float(-0.100335347731));

    ShAttrib<2, SH_TEMP, float> out;
    out = atanh(a);

    try {
      if (test.check("f_2f_callatanh_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callatanh_test5
    last_test = "f_3f_callatanh_test5";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(0.6), float(0.9));
    ShAttrib<3, SH_CONST, float> expected(float(0.549306144334), float(0.69314718056), float(1.47221948958));

    ShAttrib<3, SH_TEMP, float> out;
    out = atanh(a);

    try {
      if (test.check("f_3f_callatanh_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callatan_test0
    last_test = "f_3f_callatan_test0";
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(2.0), float(3.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.785398163397), float(1.10714871779), float(1.2490457724));

    ShAttrib<3, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_3f_callatan_test0", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callatan_test1
    last_test = "f_4f_callatan_test1";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.5), float(0.8), float(0.9));
    ShAttrib<4, SH_CONST, float> expected(float(0.291456794478), float(0.463647609001), float(0.674740942224), float(0.732815101787));

    ShAttrib<4, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_4f_callatan_test1", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callatan_test2
    last_test = "f_2f_callatan_test2";
    ShAttrib<2, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359));
    ShAttrib<2, SH_CONST, float> expected(float(1.26262725568), float(-1.26262725568));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_2f_callatan_test2", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callatan_test3
    last_test = "f_2f_callatan_test3";
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(1.41296513651), float(1.00388482185));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_2f_callatan_test3", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_callatan_test4
    last_test = "f_2f_callatan_test4";
    ShAttrib<2, SH_CONST, float> a(float(-6.28318530718), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(-1.41296513651), float(-1.00388482185));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_2f_callatan_test4", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callatan_test5
    last_test = "f_1f_callatan_test5";
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> expected(float(1.2490457724));

    ShAttrib<1, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_1f_callatan_test5", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_callatan_test6
    last_test = "f_1f_callatan_test6";
    ShAttrib<1, SH_CONST, float> a(float(0.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.0));

    ShAttrib<1, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_1f_callatan_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_callatan_test7
    last_test = "f_4f_callatan_test7";
    ShAttrib<4, SH_CONST, float> a(float(-0.5), float(-1.0), float(-3.0), float(-4.0));
    ShAttrib<4, SH_CONST, float> expected(float(-0.463647609001), float(-0.785398163397), float(-1.2490457724), float(-1.32581766367));

    ShAttrib<4, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_4f_callatan_test7", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_callatan_test8
    last_test = "f_3f_callatan_test8";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(0.463647609001), float(0.982793723247), float(1.19028994968));

    ShAttrib<3, SH_TEMP, float> out;
    out = atan(a);

    try {
      if (test.check("f_3f_callatan_test8", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_4f_callatan2_test0
    last_test = "f_4f_4f_callatan2_test0";
    ShAttrib<4, SH_CONST, float> a(float(1.0), float(6.28318530718), float(50), float(1.0));
    ShAttrib<4, SH_CONST, float> b(float(0.3), float(1.0), float(3.0), float(-6.28318530718));
    ShAttrib<4, SH_CONST, float> expected(float(1.27933953232), float(1.41296513651), float(1.51086817167), float(2.9837614633));

    ShAttrib<4, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_4f_4f_callatan2_test0", out, expected, 0.015) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_2f_callatan2_test1
    last_test = "f_2f_2f_callatan2_test1";
    ShAttrib<2, SH_CONST, float> a(float(1.0), float(1.0));
    ShAttrib<2, SH_CONST, float> b(float(1.0), float(2.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.785398163397), float(0.463647609001));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_2f_2f_callatan2_test1", out, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_2f_callatan2_test2
    last_test = "f_2f_2f_callatan2_test2";
    ShAttrib<2, SH_CONST, float> a(float(1.5), float(3.0));
    ShAttrib<2, SH_CONST, float> b(float(1.0), float(2.0));
    ShAttrib<2, SH_CONST, float> expected(float(0.982793723247), float(0.982793723247));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_2f_2f_callatan2_test2", out, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_4f_callatan2_test3
    last_test = "f_4f_4f_callatan2_test3";
    ShAttrib<4, SH_CONST, float> a(float(0), float(0), float(0), float(0));
    ShAttrib<4, SH_CONST, float> b(float(0.5), float(1.0), float(2.0), float(-1.0));
    ShAttrib<4, SH_CONST, float> expected(float(0.0), float(0.0), float(0.0), float(3.14159265359));

    ShAttrib<4, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_4f_4f_callatan2_test3", out, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_3f_callatan2_test4
    last_test = "f_3f_3f_callatan2_test4";
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> b(float(0.5), float(0.8), float(0.9));
    ShAttrib<3, SH_CONST, float> expected(float(1.10714871779), float(0.896055384571), float(0.837981225008));

    ShAttrib<3, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_3f_3f_callatan2_test4", out, expected, 0.002) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_2f_callatan2_test5
    last_test = "f_2f_2f_callatan2_test5";
    ShAttrib<2, SH_CONST, float> a(float(1.0), float(1.0));
    ShAttrib<2, SH_CONST, float> b(float(3.14159265359), float(-3.14159265359));
    ShAttrib<2, SH_CONST, float> expected(float(0.308169071116), float(2.83342358247));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_2f_2f_callatan2_test5", out, expected, 0.004) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_2f_callatan2_test6
    last_test = "f_2f_2f_callatan2_test6";
    ShAttrib<2, SH_CONST, float> a(float(3.14159265359), float(-3.14159265359));
    ShAttrib<2, SH_CONST, float> b(float(1.0), float(1.0));
    ShAttrib<2, SH_CONST, float> expected(float(1.26262725568), float(-1.26262725568));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_2f_2f_callatan2_test6", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_2f_callatan2_test7
    last_test = "f_2f_2f_callatan2_test7";
    ShAttrib<2, SH_CONST, float> a(float(6.28318530718), float(1.57079632679));
    ShAttrib<2, SH_CONST, float> b(float(-6.28318530718), float(-1.57079632679));
    ShAttrib<2, SH_CONST, float> expected(float(2.35619449019), float(2.35619449019));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_2f_2f_callatan2_test7", out, expected, 0.002) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_2f_2f_callatan2_test8
    last_test = "f_2f_2f_callatan2_test8";
    ShAttrib<2, SH_CONST, float> a(float(1.57079632679), float(1.0));
    ShAttrib<2, SH_CONST, float> b(float(1.0), float(1.0));
    ShAttrib<2, SH_CONST, float> expected(float(1.00388482185), float(0.785398163397));

    ShAttrib<2, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_2f_2f_callatan2_test8", out, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_1f_callatan2_test9
    last_test = "f_1f_1f_callatan2_test9";
    ShAttrib<1, SH_CONST, float> a(float(1.0));
    ShAttrib<1, SH_CONST, float> b(float(-1.57079632679));
    ShAttrib<1, SH_CONST, float> expected(float(2.57468114865));

    ShAttrib<1, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_1f_1f_callatan2_test9", out, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_1f_1f_callatan2_test10
    last_test = "f_1f_1f_callatan2_test10";
    ShAttrib<1, SH_CONST, float> a(float(3.0));
    ShAttrib<1, SH_CONST, float> b(float(9.0));
    ShAttrib<1, SH_CONST, float> expected(float(0.321750554397));

    ShAttrib<1, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_1f_1f_callatan2_test10", out, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_4f_callatan2_test11
    last_test = "f_4f_4f_callatan2_test11";
    ShAttrib<4, SH_CONST, float> a(float(0.5), float(-1.0), float(-3.0), float(-3.0));
    ShAttrib<4, SH_CONST, float> b(float(-1.0), float(1.0), float(1.0), float(-1.0));
    ShAttrib<4, SH_CONST, float> expected(float(2.67794504459), float(-0.785398163397), float(-1.2490457724), float(-1.89254688119));

    ShAttrib<4, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_4f_4f_callatan2_test11", out, expected, 0.01) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_3f_callatan2_test12
    last_test = "f_3f_3f_callatan2_test12";
    ShAttrib<3, SH_CONST, float> a(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> b(float(1.0), float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> expected(float(0.463647609001), float(0.982793723247), float(1.19028994968));

    ShAttrib<3, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_3f_3f_callatan2_test12", out, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_3f_3f_callatan2_test13
    last_test = "f_3f_3f_callatan2_test13";
    ShAttrib<3, SH_CONST, float> a(float(1.0), float(1.0), float(1.0));
    ShAttrib<3, SH_CONST, float> b(float(0.5), float(1.5), float(2.5));
    ShAttrib<3, SH_CONST, float> expected(float(1.10714871779), float(0.588002603548), float(0.380506377112));

    ShAttrib<3, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_3f_3f_callatan2_test13", out, expected, 0.003) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  {  // f_4f_4f_callatan2_test14
    last_test = "f_4f_4f_callatan2_test14";
    ShAttrib<4, SH_CONST, float> a(float(0.3), float(0.34), float(1.5), float(-2.5));
    ShAttrib<4, SH_CONST, float> b(float(0.0), float(0.0), float(0.0), float(0.0));
    ShAttrib<4, SH_CONST, float> expected(float(1.57079632679), float(1.57079632679), float(1.57079632679), float(-1.57079632679));

    ShAttrib<4, SH_TEMP, float> out;
    out = atan2(a, b);

    try {
      if (test.check("f_4f_4f_callatan2_test14", out, expected, 0.0001) != 0) errors++;
    } catch (const ShException &e) {
      std::cout << "Caught Sh Exception in '" << last_test << "'." << std::endl << e.message() << std::endl;
      errors++;
    } catch (const std::exception& e) {
      std::cerr << "Caught C++ exception in '" << last_test << "' : " << e.what() << std::endl;
      errors++;
    }

  }
  if (errors !=0) {
    std::cout << "Total Errors: " << errors << "/" << total_tests << std::endl;
    return 1;
  }
}

